# SVGcode 0.2
# Converts SVG paths to g-code
# (c) 2018 - Tomas 'Harvie' Mudrunka
# https://github.com/harvie
# License: GPLv2+

# Usage:
# svgcode = SVGcode('./image.svg')
# for path in svgcode.get_gcode():
#	print(path['id'])
#	print(path['path'])

import re
import numpy
import svg.path

class SVGcode:
	paths = []

	rx = {}
	rx['elements'] = re.compile('<path[^>]*>', re.IGNORECASE|re.MULTILINE)

	def __init__(self, filepath=None, string=None):
		if filepath is not None: self.read_file(filepath)
		if string is not None: self.read_string(string)

	def get_attr(self, string, attr='d'):
		if 'attr_'+attr not in self.rx.keys():
			self.rx['attr_'+attr] = re.compile('\s%s="([^">]*)"'%(attr), re.IGNORECASE|re.MULTILINE)
		m = self.rx['attr_'+attr].findall(string) or [None]
		return m[0]

	def read_string(self, data):
		#FIXME: process transform="translate(249.1743,415.5005)"
		self.paths = []
		for element in self.rx['elements'].findall(data):
			path = self.get_attr(element)
			path_id = self.get_attr(element, 'id')
			self.paths.append({'id':path_id, 'path':path})

	def read_file(self, filepath):
		with open(filepath, 'r') as myfile:
			data = myfile.read()
		return self.read_string(data)

	def path2gcode(self, path, scale=None, subdivratio=1, d=4):
		gcode = ''
		path = svg.path.parse_path(path)
		if scale is None: scale = 3.7795276 #96/25.4 #Inkscape 0.9x dots per mm

		def rv(v):
			return ('%*f'%(d,round(v/scale, d))).rstrip("0").rstrip(".")

		lastx,lasty = None,None

		for segment in path._segments:
			subdiv=max(1,round((segment.length()/scale)*subdivratio))

			if lastx != segment.start.real or lasty != segment.start.imag:
				gcode += 'G0 X%s Y%s\n'%(rv(segment.start.real),rv(-segment.start.imag))

			shape = type(segment).__name__

			if shape == 'Arc':
				if segment.sweep: garc = "G02"
				else: garc = "G03"
				center = segment.center-segment.start
				#gcode += "(arc %s %s %s %s %s %s)\n"%(segment.center,segment.radius.real,segment.arc,segment.sweep,segment.theta,segment.delta)
				#garc += ' X%s Y%s I%s J%s\n'%(rv(segment.end.real),rv(-segment.end.imag),rv(center.real),rv(-center.imag))
				gcode += '%s X%s Y%s R%s\n'%(garc, rv(segment.end.real),rv(-segment.end.imag),rv(segment.radius.real))
			elif shape in ['QuadraticBezier', 'CubicBezier', 'Arc']:
				subdiv_points = numpy.linspace(0, 1, subdiv, endpoint = False)[1:]
				for point in subdiv_points:
					gcode += 'G1 X%s Y%s\n'%(rv(segment.point(point).real),rv(-segment.point(point).imag))
				gcode += 'G1 X%s Y%s\n'%(rv(segment.end.real),rv(-segment.end.imag))
			else:
				gcode += 'G1 X%s Y%s\n'%(rv(segment.end.real),rv(-segment.end.imag))

			lastx,lasty = segment.end.real,segment.end.imag
		return gcode

	def get_gcode(self, scale=None, subdivratio=1, digits=4):
		gcode = []
		for path in self.paths:
			gcode.append({'id':path['id'], 'path':self.path2gcode(path['path'], scale, subdivratio, digits)})
		return gcode



from Sender import ControllerGeneric
import time

class Controller(ControllerGeneric):
	def __init__(self, master):
		self.master = master
		print("grbl0 loaded")

	def executeCommand(self, oline, line, cmd):
		return False

	def hardResetPre(self):
		pass

	def hardResetAfter(self):
		pass

	def viewSettings(self):
		self.master.sendGCode("$$")

	def viewBuild(self):
		self.master.sendGCode("$I")

	def viewStartup(self):
		self.master.sendGCode("$N")

	def checkGcode(self):
		self.master.sendGCode("$C")

	def grblHelp(self):
		self.master.sendGCode("$")

	def grblRestoreSettings(self):
		self.master.sendGCode("$RST=$")

	def grblRestoreWCS(self):
		self.master.sendGCode("$RST=#")

	def grblRestoreAll(self):
		self.master.sendGCode("$RST=#")

	def purgeController(self):
		time.sleep(1)
		self.master.unlock(False)




# Placed into Public Domain in June 2006 by Sean D. Spencer

# Sean D. Spencer
# sean_don4@lycos.com
# 2/19/2006
# Last Revision: 4/19/2007

# MIDI Parsing Library for Python.

TRUE = -1
FALSE = 0

class format:
    SingleTrack = 0
    MultipleTracksSync = 1
    MultipleTracksAsync = 2

class voice:
    NoteOff = 0x80
    NoteOn = 0x90
    PolyphonicKeyPressure = 0xA0 # note aftertouch
    ControllerChange = 0xB0
    ProgramChange = 0xC0
    ChannelPressure = 0xD0
    PitchBend = 0xE0

class meta:
    FileMetaEvent = 0xFF
    SMPTEOffsetMetaEvent = 0x54
    SystemExclusive = 0xF0
    SystemExclusivePacket = 0xF7
    SequenceNumber = 0x00
    TextMetaEvent = 0x01
    CopyrightMetaEvent = 0x02
    TrackName = 0x03
    InstrumentName = 0x04
    Lyric = 0x05
    Marker = 0x06
    CuePoint = 0x07
    ChannelPrefix = 0x20
    MidiPort = 0x21
    EndTrack = 0x2F
    SetTempo = 0x51
    TimeSignature = 0x58
    KeySignature = 0x59
    SequencerSpecificMetaEvent = 0x7F

class EventNote:
    def __init__(self):
	self.note_no = None
	self.velocity = None

class EventValue:
    def __init__(self):
	self.type = None
	self.value = None

class EventAmount:
    def __init__(self):
	self.amount = None

class MetaEventKeySignature:
    def __init__(self):
	self.fifths = None
	self.mode = None

class MetaEventTimeSignature:
    def __init__(self):
	self.numerator = None
	self.log_denominator = None
	self.midi_clocks = None
	self.thirty_seconds = None

class MetaEventText:
    def __init__(self):
	self.length = None
	self.text = None

class MetaEventSMPTEOffset:
    def __init__(self):
	self.hour = None
	self.minute = None
	self.second = None
	self.frame = None
	self.sub_frame = None

class MetaValues:
    def __init__(self):
	self.length = None
	self.values = None

def getNumber(theString, length):
    # MIDI uses big-endian for everything
    sum = 0
    #print "Length: " + str(length) + "  strlen: " + str(len(theString))
    for i in range(length):
        #sum = (sum *256) + int(str[i])
	sum = (sum << 8) + ord(theString[i])
    return sum, theString[length:]

def getVariableLengthNumber(str):
    sum = 0
    i = 0
    while 1:
        x = ord(str[i])
        i = i + 1
	# sum = (sum * 127) + (x (mask) 127) # mask off the 7th bit
        sum = (sum << 7) + (x & 0x7F)
	# Is 7th bit clear?
        if not (x & 0x80):
            return sum, str[i:]

def getValues(str, n=16):
    temp = []
    for x in str[:n]:
	temp.append(repr(ord(x)))
    return temp

class File:
    def __init__(self, file):
        self.file = file
        self.format = None
	self.num_tracks = None
	self.division = None
        self.tracks = []

	self.file = open(self.file, 'rb')
	str = self.file.read()
	self.file.close()

	self.read(str)

    def read(self, str):
	assert str[:4] == "MThd"
	str = str[4:]

	length, str = getNumber(str, 4)
	assert length == 6

	self.format, str = getNumber(str, 2)

	self.num_tracks, str = getNumber(str, 2)
	self.division, str = getNumber(str, 2)

	for i in range(self.num_tracks):
	    track = Track(i+1)
	    str = track.read(str)
	    self.tracks.append(track)

class Track:
    def __init__(self, index):
	self.number = index
	self.length = None
	self.events = []

    def read(self, str):
	self.length, str = getNumber(str[4:], 4)
	track_str = str[:self.length]

	prev_absolute = 0
	prev_status = 0

	i = 0
	while track_str:
	    event = Event(self.number, i+1)
	    track_str = event.read(prev_absolute, prev_status, track_str)

	    prev_absolute += event.delta
	    prev_status = event.status
	    self.events.append(event)
	    i += 1

	return str[self.length:]

class Event:
    def __init__(self, track, index):
	self.number = index
	self.type = None
	self.delta = None
	self.absolute = None
	self.status = None
	self.channel = None

    def read(self, prev_time, prev_status, str):
	self.delta, str = getVariableLengthNumber(str)
	self.absolute = prev_time + self.delta

	# use running status?
	if not (ord(str[0]) & 0x80):
	    # squeeze a duplication of the running status into the data string
	    str = prev_status + str

	self.status = str[0]
	self.channel = ord(self.status) & 0xF

	# increment one byte, past the status
	str = str[1:]

	has_channel = has_meta = TRUE

	# handle voice events
	channel_msg = ord(self.status) & 0xF0
	if channel_msg == voice.NoteOn or \
		channel_msg == voice.NoteOff or \
		channel_msg == voice.PolyphonicKeyPressure:
	    self.detail = EventNote()
	    self.detail.note_no = ord(str[0])
	    self.detail.velocity = ord(str[1])
	    str = str[2:]

	elif channel_msg == voice.ControllerChange:
	    self.detail = EventValue()
	    self.detail.type = ord(str[0])
	    self.detail.value = ord(str[1])
	    str = str[2:]

	elif channel_msg == voice.ProgramChange or \
		channel_msg == voice.ChannelPressure:

	    self.detail = EventAmount()
	    self.detail.amount = ord(str[0])
	    str = str[1:]

	elif channel_msg == voice.PitchBend:
	    # Pitch bend uses high accuracy 14 bit unsigned integer.
	    self.detail = EventAmount()
	    self.detail.amount = (ord(str[0]) << 7) | ord(str[1])
	    str = str[2:]

	else: has_channel = FALSE

	# handle meta events
	meta_msg = ord(self.status)
	if meta_msg == meta.FileMetaEvent:

	    meta_msg = type = ord(str[0])
	    length, str = getVariableLengthNumber(str[1:])

	    if type == meta.SetTempo or \
		    type == meta.ChannelPrefix:

		self.detail = EventAmount()
		self.detail.tempo, str  = getNumber(str, length)
		

	    elif type == meta.KeySignature:
		self.detail = MetaEventKeySignature()
		self.detail.fifths = ord(str[0])

		if ord(str[1]): self.detail.mode = "minor"
		else: self.detail.mode = "major"

		str = str[length:]

	    elif type == meta.TimeSignature:
		self.detail = MetaEventTimeSignature()
		self.detail.numerator = ord(str[0])
		self.detail.log_denominator = ord(str[1])
		self.detail.midi_clocks = ord(str[2])
		self.detail.thirty_seconds = ord(str[3])
		str = str[length:]

	    elif type == meta.TrackName or \
		    type == meta.TextMetaEvent or \
		    type == meta.Lyric or \
		    type == meta.CuePoint or \
		    type == meta.CopyrightMetaEvent:

		self.detail = MetaEventText()
		self.detail.length = length
		self.detail.text = str[:length]
		str = str[length:]

	    elif type == meta.SMPTEOffsetMetaEvent:
		self.detail = MetaEventSMPTEOffset()
		self.detail.hour = ord(str[0])
		self.detail.minute = ord(str[1])
		self.detail.second = ord(str[2])
		self.detail.frame = ord(str[3])
		self.detail.sub_frame = ord(str[4])
		str = str[length:]

	    elif type == meta.EndTrack:
		str = str[length:] # pass on to next track

	    else: has_meta = FALSE

	elif meta_msg == meta.SystemExclusive or \
		meta_msg == meta.SystemExclusivePacket:
	    self.detail = MetaValues()
	    self.detail.length, str = getVariableLengthNumber(str)
	    self.detail.values = getValues(str, self.detail.length)
	    str = str[self.detail.length:]

	else: has_meta = FALSE

	if has_channel:
	    self.type = channel_msg
	elif has_meta:
	    self.type = meta_msg
	else:
	    #raise "Unknown event."
            self.type = None
	return str




from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
#import enum
import math
import numpy
import logging
import collections

from python_utils import logger

from .utils import s

#: When removing empty areas, remove areas that are smaller than this
AREA_SIZE_THRESHOLD = 0
#: Vectors in a point
VECTORS = 3
#: Dimensions used in a vector
DIMENSIONS = 3


class Dimension():
    #: X index (for example, `mesh.v0[0][X]`)
    X = 0
    #: Y index (for example, `mesh.v0[0][Y]`)
    Y = 1
    #: Z index (for example, `mesh.v0[0][Z]`)
    Z = 2


# For backwards compatibility, leave the original references
X = Dimension.X
Y = Dimension.Y
Z = Dimension.Z


class RemoveDuplicates():
    '''
    Choose whether to remove no duplicates, leave only a single of the
    duplicates or remove all duplicates (leaving holes).
    '''
    NONE = 0
    SINGLE = 1
    ALL = 2

    @classmethod
    def map(cls, value):
        if value and value in cls:
            pass
        elif value:
            value = cls.SINGLE
        else:
            value = cls.NONE

        return value


def logged(class_):
    # For some reason the Logged baseclass is not properly initiated on Linux
    # systems while this works on OS X. Please let me know if you can tell me
    # what silly mistake I made here

    logger_name = logger.Logged._Logged__get_name(
        __name__,
        class_.__name__,
    )

    class_.logger = logging.getLogger(logger_name)

    for key in dir(logger.Logged):
        if not key.startswith('__'):
            setattr(class_, key, getattr(class_, key))

    return class_


@logged
class BaseMesh(logger.Logged, collections.Mapping):
    '''
    Mesh object with easy access to the vectors through v0, v1 and v2.
    The normals, areas, min, max and units are calculated automatically.

    :param numpy.array data: The data for this mesh
    :param bool calculate_normals: Whether to calculate the normals
    :param bool remove_empty_areas: Whether to remove triangles with 0 area
            (due to rounding errors for example)

    :ivar str name: Name of the solid, only exists in ASCII files
    :ivar numpy.array data: Data as :func:`BaseMesh.dtype`
    :ivar numpy.array points: All points (Nx9)
    :ivar numpy.array normals: Normals for this mesh, calculated automatically
        by default (Nx3)
    :ivar numpy.array vectors: Vectors in the mesh (Nx3x3)
    :ivar numpy.array attr: Attributes per vector (used by binary STL)
    :ivar numpy.array x: Points on the X axis by vertex (Nx3)
    :ivar numpy.array y: Points on the Y axis by vertex (Nx3)
    :ivar numpy.array z: Points on the Z axis by vertex (Nx3)
    :ivar numpy.array v0: Points in vector 0 (Nx3)
    :ivar numpy.array v1: Points in vector 1 (Nx3)
    :ivar numpy.array v2: Points in vector 2 (Nx3)

    >>> data = numpy.zeros(10, dtype=BaseMesh.dtype)
    >>> mesh = BaseMesh(data, remove_empty_areas=False)
    >>> # Increment vector 0 item 0
    >>> mesh.v0[0] += 1
    >>> mesh.v1[0] += 2

    >>> # Check item 0 (contains v0, v1 and v2)
    >>> assert numpy.array_equal(
    ...     mesh[0],
    ...     numpy.array([1., 1., 1., 2., 2., 2., 0., 0., 0.]))
    >>> assert numpy.array_equal(
    ... mesh.vectors[0],
    ... numpy.array([[1., 1., 1.],
    ...     [2., 2., 2.],
    ...     [0., 0., 0.]]))
    >>> assert numpy.array_equal(
    ...     mesh.v0[0],
    ...     numpy.array([1., 1., 1.]))
    >>> assert numpy.array_equal(
    ...     mesh.points[0],
    ...     numpy.array([1., 1., 1., 2., 2., 2., 0., 0., 0.]))
    >>> assert numpy.array_equal(
    ...     mesh.data[0],
    ...     numpy.array((
    ...             [0., 0., 0.],
    ...             [[1., 1., 1.], [2., 2., 2.], [0., 0., 0.]],
    ...             [0]),
    ...         dtype=BaseMesh.dtype))
    >>> assert numpy.array_equal(mesh.x[0], numpy.array([1., 2., 0.]))

    >>> mesh[0] = 3
    >>> assert numpy.array_equal(
    ...     mesh[0],
    ...     numpy.array([3., 3., 3., 3., 3., 3., 3., 3., 3.]))

    >>> len(mesh) == len(list(mesh))
    True
    >>> (mesh.min_ < mesh.max_).all()
    True
    >>> mesh.update_normals()
    >>> mesh.units.sum()
    0.0
    >>> mesh.v0[:] = mesh.v1[:] = mesh.v2[:] = 0
    >>> mesh.points.sum()
    0.0

    >>> mesh.v0 = mesh.v1 = mesh.v2 = 0
    >>> mesh.x = mesh.y = mesh.z = 0

    >>> mesh.attr = 1
    >>> (mesh.attr == 1).all()
    True

    >>> mesh.normals = 2
    >>> (mesh.normals == 2).all()
    True

    >>> mesh.vectors = 3
    >>> (mesh.vectors == 3).all()
    True

    >>> mesh.points = 4
    >>> (mesh.points == 4).all()
    True
    '''
    #: - normals: :func:`numpy.float32`, `(3, )`
    #: - vectors: :func:`numpy.float32`, `(3, 3)`
    #: - attr: :func:`numpy.uint16`, `(1, )`
    dtype = numpy.dtype([
        (s('normals'), numpy.float32, (3, )),
        (s('vectors'), numpy.float32, (3, 3)),
        (s('attr'), numpy.uint16, (1, )),
    ])
    dtype = dtype.newbyteorder('<')  # Even on big endian arches, use little e.

    def __init__(self, data, calculate_normals=True,
                 remove_empty_areas=False,
                 remove_duplicate_polygons=RemoveDuplicates.NONE,
                 name='', speedups=True, **kwargs):
        super(BaseMesh, self).__init__(**kwargs)
        self.speedups = speedups
        if remove_empty_areas:
            data = self.remove_empty_areas(data)

        if RemoveDuplicates.map(remove_duplicate_polygons):
            data = self.remove_duplicate_polygons(data,
                                                  remove_duplicate_polygons)

        self.name = name
        self.data = data

        if calculate_normals:
            self.update_normals()

    @property
    def attr(self):
        return self.data['attr']

    @attr.setter
    def attr(self, value):
        self.data['attr'] = value

    @property
    def normals(self):
        return self.data['normals']

    @normals.setter
    def normals(self, value):
        self.data['normals'] = value

    @property
    def vectors(self):
        return self.data['vectors']

    @vectors.setter
    def vectors(self, value):
        self.data['vectors'] = value

    @property
    def points(self):
        return self.vectors.reshape(self.data.size, 9)

    @points.setter
    def points(self, value):
        self.points[:] = value

    @property
    def v0(self):
        return self.vectors[:, 0]

    @v0.setter
    def v0(self, value):
        self.vectors[:, 0] = value

    @property
    def v1(self):
        return self.vectors[:, 1]

    @v1.setter
    def v1(self, value):
        self.vectors[:, 1] = value

    @property
    def v2(self):
        return self.vectors[:, 2]

    @v2.setter
    def v2(self, value):
        self.vectors[:, 2] = value

    @property
    def x(self):
        return self.points[:, Dimension.X::3]

    @x.setter
    def x(self, value):
        self.points[:, Dimension.X::3] = value

    @property
    def y(self):
        return self.points[:, Dimension.Y::3]

    @y.setter
    def y(self, value):
        self.points[:, Dimension.Y::3] = value

    @property
    def z(self):
        return self.points[:, Dimension.Z::3]

    @z.setter
    def z(self, value):
        self.points[:, Dimension.Z::3] = value

    @classmethod
    def remove_duplicate_polygons(cls, data, value=RemoveDuplicates.SINGLE):
        value = RemoveDuplicates.map(value)
        polygons = data['vectors'].sum(axis=1)
        # Get a sorted list of indices
        idx = numpy.lexsort(polygons.T)
        # Get the indices of all different indices
        diff = numpy.any(polygons[idx[1:]] != polygons[idx[:-1]], axis=1)

        if value is RemoveDuplicates.SINGLE:
            # Only return the unique data, the True is so we always get at
            # least the originals
            return data[numpy.sort(idx[numpy.concatenate(([True], diff))])]
        elif value is RemoveDuplicates.ALL:
            # We need to return both items of the shifted diff
            diff_a = numpy.concatenate(([True], diff))
            diff_b = numpy.concatenate((diff, [True]))
            diff = numpy.concatenate((diff, [False]))

            # Combine both unique lists
            filtered_data = data[numpy.sort(idx[diff_a & diff_b])]
            if len(filtered_data) <= len(data) / 2:
                return data[numpy.sort(idx[diff_a])]
            else:
                return data[numpy.sort(idx[diff])]
        else:
            return data

    @classmethod
    def remove_empty_areas(cls, data):
        vectors = data['vectors']
        v0 = vectors[:, 0]
        v1 = vectors[:, 1]
        v2 = vectors[:, 2]
        normals = numpy.cross(v1 - v0, v2 - v0)
        squared_areas = (normals ** 2).sum(axis=1)
        return data[squared_areas > AREA_SIZE_THRESHOLD ** 2]

    def update_normals(self):
        '''Update the normals for all points'''
        self.normals[:] = numpy.cross(self.v1 - self.v0, self.v2 - self.v0)

    def update_min(self):
        self._min = self.vectors.min(axis=(0, 1))

    def update_max(self):
        self._max = self.vectors.max(axis=(0, 1))

    def update_areas(self):
        areas = .5 * numpy.sqrt((self.normals ** 2).sum(axis=1))
        self.areas = areas.reshape((areas.size, 1))

    def check(self):
        if (self.normals.sum(axis=0) >= 1e-4).any():
            self.warning('''
            Your mesh is not closed, the mass methods will not function
            correctly on this mesh.  For more info:
            https://github.com/WoLpH/numpy-stl/issues/69
            '''.strip())
            return False
        else:
            return True

    def get_mass_properties(self):
        '''
        Evaluate and return a tuple with the following elements:
          - the volume
          - the position of the center of gravity (COG)
          - the inertia matrix expressed at the COG

        Documentation can be found here:
        http://www.geometrictools.com/Documentation/PolyhedralMassProperties.pdf
        '''
        self.check()

        def subexpression(x):
            w0, w1, w2 = x[:, 0], x[:, 1], x[:, 2]
            temp0 = w0 + w1
            f1 = temp0 + w2
            temp1 = w0 * w0
            temp2 = temp1 + w1 * temp0
            f2 = temp2 + w2 * f1
            f3 = w0 * temp1 + w1 * temp2 + w2 * f2
            g0 = f2 + w0 * (f1 + w0)
            g1 = f2 + w1 * (f1 + w1)
            g2 = f2 + w2 * (f1 + w2)
            return f1, f2, f3, g0, g1, g2

        x0, x1, x2 = self.x[:, 0], self.x[:, 1], self.x[:, 2]
        y0, y1, y2 = self.y[:, 0], self.y[:, 1], self.y[:, 2]
        z0, z1, z2 = self.z[:, 0], self.z[:, 1], self.z[:, 2]
        a1, b1, c1 = x1 - x0, y1 - y0, z1 - z0
        a2, b2, c2 = x2 - x0, y2 - y0, z2 - z0
        d0, d1, d2 = b1 * c2 - b2 * c1, a2 * c1 - a1 * c2, a1 * b2 - a2 * b1

        f1x, f2x, f3x, g0x, g1x, g2x = subexpression(self.x)
        f1y, f2y, f3y, g0y, g1y, g2y = subexpression(self.y)
        f1z, f2z, f3z, g0z, g1z, g2z = subexpression(self.z)

        intg = numpy.zeros((10))
        intg[0] = sum(d0 * f1x)
        intg[1:4] = sum(d0 * f2x), sum(d1 * f2y), sum(d2 * f2z)
        intg[4:7] = sum(d0 * f3x), sum(d1 * f3y), sum(d2 * f3z)
        intg[7] = sum(d0 * (y0 * g0x + y1 * g1x + y2 * g2x))
        intg[8] = sum(d1 * (z0 * g0y + z1 * g1y + z2 * g2y))
        intg[9] = sum(d2 * (x0 * g0z + x1 * g1z + x2 * g2z))
        intg /= numpy.array([6, 24, 24, 24, 60, 60, 60, 120, 120, 120])
        volume = intg[0]
        cog = intg[1:4] / volume
        cogsq = cog ** 2
        inertia = numpy.zeros((3, 3))
        inertia[0, 0] = intg[5] + intg[6] - volume * (cogsq[1] + cogsq[2])
        inertia[1, 1] = intg[4] + intg[6] - volume * (cogsq[2] + cogsq[0])
        inertia[2, 2] = intg[4] + intg[5] - volume * (cogsq[0] + cogsq[1])
        inertia[0, 1] = inertia[1, 0] = -(intg[7] - volume * cog[0] * cog[1])
        inertia[1, 2] = inertia[2, 1] = -(intg[8] - volume * cog[1] * cog[2])
        inertia[0, 2] = inertia[2, 0] = -(intg[9] - volume * cog[2] * cog[0])
        return volume, cog, inertia

    def update_units(self):
        units = self.normals.copy()
        non_zero_areas = self.areas > 0
        areas = self.areas

        if non_zero_areas.shape[0] != areas.shape[0]:  # pragma: no cover
            self.warning('Zero sized areas found, '
                         'units calculation will be partially incorrect')

        if non_zero_areas.any():
            non_zero_areas.shape = non_zero_areas.shape[0]
            areas = numpy.hstack((2 * areas[non_zero_areas],) * DIMENSIONS)
            units[non_zero_areas] /= areas

        self.units = units

    @classmethod
    def rotation_matrix(cls, axis, theta):
        '''
        Generate a rotation matrix to Rotate the matrix over the given axis by
        the given theta (angle)

        Uses the `Euler-Rodrigues
        <https://en.wikipedia.org/wiki/Euler%E2%80%93Rodrigues_formula>`_
        formula for fast rotations.

        :param numpy.array axis: Axis to rotate over (x, y, z)
        :param float theta: Rotation angle in radians, use `math.radians` to
                     convert degrees to radians if needed.
        '''
        axis = numpy.asarray(axis)
        # No need to rotate if there is no actual rotation
        if not axis.any():
            return numpy.zeros((3, 3))

        theta = 0.5 * numpy.asarray(theta)

        axis = axis / numpy.linalg.norm(axis)

        a = math.cos(theta)
        b, c, d = - axis * math.sin(theta)
        angles = a, b, c, d
        powers = [x * y for x in angles for y in angles]
        aa, ab, ac, ad = powers[0:4]
        ba, bb, bc, bd = powers[4:8]
        ca, cb, cc, cd = powers[8:12]
        da, db, dc, dd = powers[12:16]

        return numpy.array([[aa + bb - cc - dd, 2 * (bc + ad), 2 * (bd - ac)],
                            [2 * (bc - ad), aa + cc - bb - dd, 2 * (cd + ab)],
                            [2 * (bd + ac), 2 * (cd - ab), aa + dd - bb - cc]])

    def rotate(self, axis, theta=0, point=None):
        '''
        Rotate the matrix over the given axis by the given theta (angle)

        Uses the :py:func:`rotation_matrix` in the background.

        .. note:: Note that the `point` was accidentaly inverted with the
           old version of the code. To get the old and incorrect behaviour
           simply pass `-point` instead of `point` or `-numpy.array(point)` if
           you're passing along an array.

        :param numpy.array axis: Axis to rotate over (x, y, z)
        :param float theta: Rotation angle in radians, use `math.radians` to
                            convert degrees to radians if needed.
        :param numpy.array point: Rotation point so manual translation is not
                                  required
        '''
        # No need to rotate if there is no actual rotation
        if not theta:
            return

        self.rotate_using_matrix(self.rotation_matrix(axis, theta), point)

    def rotate_using_matrix(self, rotation_matrix, point=None):
        # No need to rotate if there is no actual rotation
        if not rotation_matrix.any():
            return

        if isinstance(point, (numpy.ndarray, list, tuple)) and len(point) == 3:
            point = numpy.asarray(point)
        elif point is None:
            point = numpy.array([0, 0, 0])
        elif isinstance(point, (int, float)):
            point = numpy.asarray([point] * 3)
        else:
            raise TypeError('Incorrect type for point', point)

        def _rotate(matrix):
            if point.any():
                # Translate while rotating
                return (matrix - point).dot(rotation_matrix) + point
            else:
                # Simply apply the rotation
                return matrix.dot(rotation_matrix)

        for i in range(3):
            self.vectors[:, i] = _rotate(self.vectors[:, i])

    def translate(self, translation):
        '''
        Translate the mesh in the three directions

        :param numpy.array translation: Translation vector (x, y, z)
        '''
        assert len(translation) == 3, "Translation vector must be of length 3"
        self.x += translation[0]
        self.y += translation[1]
        self.z += translation[2]

    def transform(self, matrix):
        '''
        Transform the mesh with a rotation and a translation stored in a
        single 4x4 matrix

        :param numpy.array matrix: Transform matrix with shape (4, 4), where
                                   matrix[0:3, 0:3] represents the rotation
                                   part of the transformation
                                   matrix[0:3, 3] represents the translation
                                   part of the transformation
        '''
        is_a_4x4_matrix = matrix.shape == (4, 4)
        assert is_a_4x4_matrix, "Transformation matrix must be of shape (4, 4)"
        rotation = matrix[0:3, 0:3]
        unit_det_rotation = numpy.allclose(numpy.linalg.det(rotation), 1.0)
        assert unit_det_rotation, "Rotation matrix has not a unit determinant"
        for i in range(3):
            self.vectors[:, i] = numpy.dot(rotation, self.vectors[:, i].T).T
        self.x += matrix[0, 3]
        self.y += matrix[1, 3]
        self.z += matrix[2, 3]

    def _get_or_update(key):
        def _get(self):
            if not hasattr(self, '_%s' % key):
                getattr(self, 'update_%s' % key)()
            return getattr(self, '_%s' % key)

        return _get

    def _set(key):
        def _set(self, value):
            setattr(self, '_%s' % key, value)

        return _set

    min_ = property(_get_or_update('min'), _set('min'),
                    doc='Mesh minimum value')
    max_ = property(_get_or_update('max'), _set('max'),
                    doc='Mesh maximum value')
    areas = property(_get_or_update('areas'), _set('areas'),
                     doc='Mesh areas')
    units = property(_get_or_update('units'), _set('units'),
                     doc='Mesh unit vectors')

    def __getitem__(self, k):
        return self.points[k]

    def __setitem__(self, k, v):
        self.points[k] = v

    def __len__(self):
        return self.points.shape[0]

    def __iter__(self):
        for point in self.points:
            yield point





from Sender import ControllerGeneric
import time

class Controller(ControllerGeneric):
	def __init__(self, master):
		self.master = master
		print("smoothie loaded")

	def executeCommand(self, oline, line, cmd):
		if line[0] in ( "help", "version", "mem", "ls",
				"cd", "pwd", "cat", "rm", "mv",
				"remount", "play", "progress", "abort",
				"reset", "dfu", "break", "config-get",
				"config-set", "get", "set_temp", "get",
				"get", "net", "load", "save", "upload",
				"calc_thermistor", "thermistors", "md5sum"):
			self.master.serial.write(oline+"\n")
			return True
		return False

	def hardResetPre(self):
		self.master.serial.write(b"reset\n")

	def hardResetAfter(self):
		time.sleep(6)

	def viewSettings(self):
		pass

	def viewBuild(self):
		self.master.serial.write(b"version\n")
		self.master.sendGCode("$I")

	def viewStartup(self):
		pass

	def checkGcode(self):
		pass

	def grblHelp(self):
		self.master.serial.write(b"help\n")

	def grblRestoreSettings(self):
		pass

	def grblRestoreWCS(self):
		pass

	def grblRestoreAll(self):
		pass

	def purgeController(self):
		pass



# -*- coding: ascii -*-
# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2007 Alex Holkner
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#	notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#	notice, this list of conditions and the following disclaimer in
#	the documentation and/or other materials provided with the
#	distribution.
#  * Neither the name of the pyglet nor the names of its
#	contributors may be used to endorse or promote products
#	derived from this software without specific prior written
#	permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
#
#Implementation of the Truetype file format.
#
#References:
# * http://developer.apple.com/fonts/TTRefMan/RM06
# * http://www.microsoft.com/typography/otspec
#
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

#
# Deriver from Alex Holkner work for pyglet
# Glyph data ported from
# * http://stevehanov.ca/blog/index.php?id=143
# the JavaScript code to extract also Glyph data as vector.
# To extract contours out of ttf structure data, here are some other useful links:
#  * http://chanae.walon.org/pub/ttf/ttf_glyphs.htm
#  * http://freetype.sourceforge.net/freetype2/docs/glyphs/glyphs-6.html#section-1
#
#   Filippo Rivato f.rivato@gmail.com

import codecs
import os
import mmap
import struct

class TruetypeInfo:
	"""Information about a single Truetype face.

	The class memory-maps the font file to read the tables, so
	it is vital that you call the `close` method to avoid large memory
	leaks.  Once closed, you cannot call any of the ``get_*`` methods.
	"""

	_name_id_lookup = {
		'copyright': 0,
		'family': 1,
		'subfamily': 2,
		'identifier': 3,
		'name': 4,
		'version': 5,
		'postscript': 6,
		'trademark': 7,
		'manufacturer': 8,
		'designer': 9,
		'description': 10,
		'vendor-url': 11,
		'designer-url': 12,
		'license': 13,
		'license-url': 14,
		'preferred-family': 16,
		'preferred-subfamily': 17,
		'compatible-name': 18,
		'sample': 19,
	}

	_platform_id_lookup = {
		'unicode': 0,
		'macintosh': 1,
		'iso': 2,
		'microsoft': 3,
		'custom': 4
	}

	_microsoft_encoding_lookup = {
		1: 'utf_16_be',
		2: 'shift_jis',
		4: 'big5',
		6: 'johab',
		10: 'utf_16_be'
	}

	_macintosh_encoding_lookup = {
		0: 'mac_roman'
	}

	def __init__(self, filename):
		"""Read the given truetype file.

		:Parameters:
			`filename`
				The name of any Windows, OS2 or Macintosh Truetype file.

		The object must be closed (see `close`) after use.

		An exception will be raised if the file does not exist or cannot
		be read.
		"""
		if not filename: filename = ''
		len = os.stat(filename).st_size
		self._fileno = os.open(filename, os.O_RDONLY)
		if hasattr(mmap, 'MAP_SHARED'):
			self._data = mmap.mmap(self._fileno, len, mmap.MAP_SHARED,
				mmap.PROT_READ)
		else:
			self._data = mmap.mmap(self._fileno, len, None, mmap.ACCESS_READ)

		offsets = _read_offset_table(self._data, 0)
		self._tables = {}
		for table in _read_table_directory_entry.array(self._data,
			offsets.size, offsets.num_tables):
			self._tables[table.tag] = table

		self._names = None
		self._horizontal_metrics = None
		self._character_advances = None
		self._character_kernings = None
		self._glyph_kernings = None
		self._character_map = None
		self._glyph_map = None
		self._font_selection_flags = None
		self._glyph_vectors = None

		self.header = \
			_read_head_table(self._data, self._tables['head'].offset)
		self.horizontal_header = \
			_read_horizontal_header(self._data, self._tables['hhea'].offset)

	def get_font_selection_flags(self):
		"""Return the font selection flags, as defined in OS/2 table"""
		if not self._font_selection_flags:
			OS2_table = \
				_read_OS2_table(self._data, self._tables['OS/2'].offset)
			self._font_selection_flags = OS2_table.fs_selection
		return self._font_selection_flags

	def is_bold(self):
		"""Returns True iff the font describes itself as bold."""
		return bool(self.get_font_selection_flags() & 0x20)

	def is_italic(self):
		"""Returns True iff the font describes itself as italic."""
		return bool(self.get_font_selection_flags() & 0x1)

	def get_names(self):
		"""Returns a dictionary of names defined in the file.

		The key of each item is a tuple of ``platform_id``, ``name_id``,
		where each ID is the number as described in the Truetype format.

		The value of each item is a tuple of
		``encoding_id``, ``language_id``, ``value``, where ``value`` is
		an encoded string.
		"""
		if self._names:
			return self._names
		naming_table = \
			_read_naming_table(self._data, self._tables['name'].offset)
		name_records = \
			_read_name_record.array(self._data,
				self._tables['name'].offset + naming_table.size,
				naming_table.count)
		storage = naming_table.string_offset + self._tables['name'].offset
		self._names = {}
		for record in name_records:
			value = self._data[record.offset + storage:\
							   record.offset + storage + record.length]
			key = record.platform_id, record.name_id
			value = (record.encoding_id, record.language_id, value)
			if not key in self._names:
				self._names[key] = []
			self._names[key].append(value)
		return self._names

	def get_name(self, name, platform=None, languages=None):
		"""Returns the value of the given name in this font.

		:Parameters:
			`name`
				Either an integer, representing the name_id desired (see
				font format); or a string describing it, see below for
				valid names.
			`platform`
				Platform for the requested name.  Can be the integer ID,
				or a string describing it.  By default, the Microsoft
				platform is searched first, then Macintosh.
			`languages`
				A list of language IDs to search.  The first language
				which defines the requested name will be used.  By default,
				all English dialects are searched.

		If the name is not found, ``None`` is returned.  If the name
		is found, the value will be decoded and returned as a unicode
		string.  Currently only some common encodings are supported.

		Valid names to request are (supply as a string)::

			'copyright'
			'family'
			'subfamily'
			'identifier'
			'name'
			'version'
			'postscript'
			'trademark'
			'manufacturer'
			'designer'
			'description'
			'vendor-url'
			'designer-url'
			'license'
			'license-url'
			'preferred-family'
			'preferred-subfamily'
			'compatible-name'
			'sample'

		Valid platforms to request are (supply as a string)::

			'unicode'
			'macintosh'
			'iso'
			'microsoft'
			'custom'
		"""

		names = self.get_names()
		if type(name) == str:
			name = self._name_id_lookup[name]
		if not platform:
			for platform in ('microsoft','macintosh'):
				value = self.get_name(name, platform, languages)
				if value:
					return value
		if type(platform) == str:
			platform = self._platform_id_lookup[platform]
		if not (platform, name) in names:
			return None

		if platform == 3: # setup for microsoft
			encodings = self._microsoft_encoding_lookup
			if not languages:
				# Default to english languages for microsoft
				languages = (0x409,0x809,0xc09,0x1009,0x1409,0x1809)
		elif platform == 1: # setup for macintosh
			encodings = self.__macintosh_encoding_lookup
			if not languages:
				# Default to english for macintosh
				languages = (0,)

		for record in names[(platform, name)]:
			if record[1] in languages and record[0] in encodings:
				decoder = codecs.getdecoder(encodings[record[0]])
				return decoder(record[2])[0]
		return None

	def get_horizontal_metrics(self):
		"""Return all horizontal metric entries in table format."""
		if not self._horizontal_metrics:
			ar = _read_long_hor_metric.array(self._data,
				 self._tables['hmtx'].offset,
				 self.horizontal_header.number_of_h_metrics)
			self._horizontal_metrics = ar
		return self._horizontal_metrics

	def get_character_advances(self):
		"""Return a dictionary of character->advance.

		They key of the dictionary is a unit-length unicode string,
		and the value is a float giving the horizontal advance in
		em.
		"""
		if self._character_advances:
			return self._character_advances
		ga = self.get_glyph_advances()
		gmap = self.get_glyph_map()
		self._character_advances = {}
		for i in range(len(ga)):
			if i in gmap and not gmap[i] in self._character_advances:
				self._character_advances[gmap[i]] = ga[i]
		return self._character_advances

	def get_glyph_advances(self):
		"""Return a dictionary of glyph->advance.

		They key of the dictionary is the glyph index and the value is a float
		giving the horizontal advance in em.
		"""
		hm = self.get_horizontal_metrics()
		return [float(m.advance_width) / self.header.units_per_em for m in hm]

	def get_character_kernings(self):
		"""Return a dictionary of (left,right)->kerning

		The key of the dictionary is a tuple of ``(left, right)``
		where each element is a unit-length unicode string.  The
		value of the dictionary is the horizontal pairwise kerning
		in em.
		"""
		if not self._character_kernings:
			gmap = self.get_glyph_map()
			kerns = self.get_glyph_kernings()
			self._character_kernings = {}
			for pair, value in kerns.items():
				lglyph, rglyph = pair
				lchar = lglyph in gmap and gmap[lglyph] or None
				rchar = rglyph in gmap and gmap[rglyph] or None
				if lchar and rchar:
					self._character_kernings[(lchar, rchar)] = value
		return self._character_kernings

	def get_glyph_kernings(self):
		"""Return a dictionary of (left,right)->kerning

		The key of the dictionary is a tuple of ``(left, right)``
		where each element is a glyph index.  The value of the dictionary is
		the horizontal pairwise kerning in em.
		"""
		if self._glyph_kernings:
			return self._glyph_kernings
		header = \
			_read_kern_header_table(self._data, self._tables['kern'].offset)
		offset = self._tables['kern'].offset + header.size
		kernings = {}
		for i in range(header.n_tables):
			header = _read_kern_subtable_header(self._data, offset)
			if header.coverage & header.horizontal_mask \
			   and not header.coverage & header.minimum_mask \
			   and not header.coverage & header.perpendicular_mask:
				if header.coverage & header.format_mask == 0:
					self._add_kernings_format0(kernings, offset + header.size)
			offset += header.length
		self._glyph_kernings = kernings
		return kernings

	def _add_kernings_format0(self, kernings, offset):
		header = _read_kern_subtable_format0(self._data, offset)
		kerning_pairs = _read_kern_subtable_format0Pair.array(self._data,
			offset + header.size, header.n_pairs)
		for pair in kerning_pairs:
			if (pair.left, pair.right) in kernings:
				kernings[(pair.left, pair.right)] += pair.value \
					/ float(self.header.units_per_em)
			else:
				kernings[(pair.left, pair.right)] = pair.value \
					/ float(self.header.units_per_em)

	def get_glyph_map(self):
		"""Calculate and return a reverse character map.

		Returns a dictionary where the key is a glyph index and the
		value is a unit-length unicode string.
		"""
		if self._glyph_map:
			return self._glyph_map
		cmap = self.get_character_map()
		self._glyph_map = {}
		for ch, glyph in cmap.items():
			if not glyph in self._glyph_map:
				self._glyph_map[glyph] = ch
		return self._glyph_map

	def get_character_map(self):
		"""Return the character map.

		Returns a dictionary where the key is a unit-length unicode
		string and the value is a glyph index.  Currently only
		format 4 character maps are read.
		"""
		if self._character_map:
			return self._character_map
		cmap = _read_cmap_header(self._data, self._tables['cmap'].offset)
		records = _read_cmap_encoding_record.array(self._data,
			self._tables['cmap'].offset + cmap.size, cmap.num_tables)
		self._character_map = {}
		for record in records:
			if record.platform_id == 3 and record.encoding_id == 1:
				# Look at Windows Unicode charmaps only
				offset = self._tables['cmap'].offset + record.offset
				format_header = _read_cmap_format_header(self._data, offset)
				if format_header.format == 4:
					self._character_map = \
						self._get_character_map_format4(offset)
					break
		return self._character_map

	def _get_character_map_format4(self, offset):
		# This is absolutely, without question, the *worst* file
		# format ever.  Whoever the fuckwit is that thought this up is
		# a fuckwit.
		header = _read_cmap_format4Header(self._data, offset)
		seg_count = header.seg_count_x2 / 2
		array_size = struct.calcsize('>%dH' % seg_count)
		end_count = self._read_array('>%dH' % seg_count,
			offset + header.size)
		start_count = self._read_array('>%dH' % seg_count,
			offset + header.size + array_size + 2)
		id_delta = self._read_array('>%dh' % seg_count,
			offset + header.size + array_size + 2 + array_size)
		id_range_offset_address = \
			offset + header.size + array_size + 2 + array_size + array_size
		id_range_offset = self._read_array('>%dH' % seg_count,
			id_range_offset_address)
		character_map = {}
		for i in range(0, seg_count):
			if id_range_offset[i] != 0:
				if id_range_offset[i] == 65535:
					continue  # Hack around a dodgy font (babelfish.ttf)
				for c in range(start_count[i], end_count[i] + 1):
					addr = id_range_offset[i] + 2 * (c - start_count[i]) + \
						id_range_offset_address + 2 * i
					g = struct.unpack('>H', self._data[addr:addr + 2])[0]
					if g != 0:
						character_map[unichr(c)] = (g + id_delta[i]) % 65536
			else:
				for c in range(start_count[i], end_count[i] + 1):
					g = (c + id_delta[i]) % 65536
					if g != 0:
						character_map[unichr(c)] = g
		return character_map

	def _get_glyph_offset(self, index):
		loca = self._tables['loca']
		if self.header.index_to_loc_format == 1 :
			fmt = ">I" #uint32
			mul = 1
		else:
			fmt = ">H" #uint16
			mul = 2
		size = struct.calcsize(fmt)
		offset = loca.offset + (size * index)
		res = struct.unpack(fmt, self._data[offset:offset + size])[0] * mul
		return res + self._tables["glyf"].offset

	def _read_glyph(self,index):
		g_offset = self._get_glyph_offset(index)
		g_size_table = _read_glyph_size_table.array(self._data,
								g_offset,
								1)[0]
		g_offset += g_size_table.size

		if (g_size_table.numContours == -1):
			glyph = self._read_compound_glyp(g_size_table,g_offset)
		else:
			glyph = self._read_simple_glyp(g_size_table,g_offset)
		return glyph

	def _read_simple_glyp(self,glyph_size,g_offset):
		ON_CURVE = 1
		X_IS_BYTE = 2
		Y_IS_BYTE = 4
		REPEAT = 8
		X_DELTA = 16
		Y_DELTA = 32

		glyph = Glyph()
		glyph.type = "simple"
		glyph.contoursEnd = []
		glyph.points = []
		#glyph.size = glyph_size

		#add contours end
		for i in range(0,glyph_size.numContours):
			ce,g_offset = self._get_data(">H", g_offset)#uint16
			glyph.contoursEnd.append(ce)

		#skip over intructions
		seek,g_offset = self._get_data(">H", g_offset)#uint16
		g_offset += seek

		if (glyph_size.numContours == 0): return

		numPoints = max(glyph.contoursEnd) + 1
		flags = []

		i = 0
		while (i < numPoints):
			flag, g_offset = self._get_data(">B", g_offset)#uint8
			flags.append(flag)
			gp = GlyphPoint()
			gp.ON_CURVE = (flag & ON_CURVE) > 0
			glyph.points.append(gp)

			if (flag & REPEAT) :
				repeat_count,g_offset = self._get_data(">B", g_offset)#uint8
				i += repeat_count
				while (repeat_count > 0):
					flags.append(flag)
					gp = GlyphPoint()
					gp.ON_CURVE = (flag & ON_CURVE) > 0
					glyph.points.append(gp)
					repeat_count-=1
			i += 1

		#c = len(flags)
		#Parse for xValues
		xValue = 0
		for i,f in enumerate(flags):
			if (f & X_IS_BYTE > 0):
				vx,g_offset = self._get_data(">B", g_offset)#uint8
				if (f & X_DELTA) > 0:
					xValue += vx
				else:
					xValue -= vx
			elif (-(f + 1) & X_DELTA) > 0: #??????????  else if ( ~flag & deltaFlag ) -(N+1)
				vx,g_offset = self._get_data(">h", g_offset)#int16
				xValue += vx
			else:
				pass #value unchage
			#Save value
			glyph.points[i].x = xValue

		#Parse for yValues
		yValue = 0
		for i,f in enumerate(flags):
			if (f & Y_IS_BYTE > 0):
				vy,g_offset = self._get_data(">B", g_offset)#uint8
				if (f & Y_DELTA) > 0:
					yValue += vy
				else:
					yValue -= vy
			elif (-(f + 1) & Y_DELTA) > 0: #??????????  else if ( ~flag & deltaFlag )
				vy,g_offset = self._get_data(">h", g_offset)#int16
				yValue += vy
			else:
				pass #value unchage
			#Save value
			glyph.points[i].y = yValue
		return glyph

	def _get_data(self, fmt, g_offset):
		size = struct.calcsize(fmt)
		data = struct.unpack(fmt, self._data[g_offset:g_offset + size])[0]
		g_offset += size
		return data,g_offset

	def _read_compound_glyp(self,glyph_size,g_offset):
		#FIXME:implements extraction of data for complex glyph
		ARG_1_AND_2_ARE_WORDS    = 1
		ARGS_ARE_XY_VALUES       = 2
		ROUND_XY_TO_GRID         = 4
		WE_HAVE_A_SCALE          = 8
		RESERVED                 = 16
		MORE_COMPONENTS          = 32
		WE_HAVE_AN_X_AND_Y_SCALE = 64
		WE_HAVE_A_TWO_BY_TWO     = 128
		WE_HAVE_INSTRUCTIONS     = 256
		USE_MY_METRICS           = 512
		OVERLAP_COMPONENT        = 1024

		glyph = Glyph()
		glyph.type = "compound"
		glyph.components = []

		flags =  MORE_COMPONENTS
		while flags & MORE_COMPONENTS:
			flags,g_offset = self._get_data(">H", g_offset)#uint16

			component = GlyphComponent()
			component.glyphIndex,g_offset = self._get_data(">H", g_offset)#uint16

			arg1 = 0
			arg2 = 0
			if ( flags &  ARG_1_AND_2_ARE_WORDS ):
				arg1,g_offset = self._get_data(">H", g_offset)#uint16
				arg2,g_offset = self._get_data(">H", g_offset)#uint16
			else:
				arg1,g_offset = self._get_data(">B", g_offset)#uint8
				arg2,g_offset = self._get_data(">B", g_offset)#uint8

			if ( flags & ARGS_ARE_XY_VALUES ):
				component.e = arg1
				component.f = arg2
			else:
				component.destPointIndex = arg1
				component.srcPointIndex = arg2

			if ( flags & WE_HAVE_A_SCALE ):
				component.a,g_offset = self.get2Dot14(g_offset)
				component.d = component.a
			elif( flags & WE_HAVE_AN_X_AND_Y_SCALE ):
				component.a,g_offset = self.get2Dot14(g_offset)
				component.d,g_offset = self.get2Dot14(g_offset)
			elif( flags & WE_HAVE_A_TWO_BY_TWO ):
				component.a,g_offset = self.get2Dot14(g_offset)
				component.b,g_offset = self.get2Dot14(g_offset)
				component.c,g_offset = self.get2Dot14(g_offset)
				component.d,g_offset = self.get2Dot14(g_offset)

			#add component
			glyph.components.append(component)

			if ( flags & WE_HAVE_INSTRUCTIONS ):
				seek,g_offset = self._get_data(">H", g_offset)#uint16
				g_offset += seek

		#Create glyph points
		ceOffset = 0
		for component in glyph.components:
#			print component.glyphIndex
			subGlyph = self._read_glyph(component.glyphIndex)
			#apply transformation to points
			#FIX ME: not all transformations (es. scale) are applied correctly
			for p in subGlyph.points:
				tp = GlyphPoint(p.x,p.y)
				tp.ON_CURVE = p.ON_CURVE
				tp.x += component.e
				tp.y += component.f
				glyph.points.append(tp)
			#append contours ends
			for ce in subGlyph.contoursEnd:
				glyph.contoursEnd.append(ceOffset+ce)
			ceOffset += len(subGlyph.points)
		return glyph


	def get2Dot14(self,g_offset):
		v,g_offset = self._get_data(">h", g_offset)#int16
		return v / (1 << 14),g_offset

	def get_glyph_vector(self,index):
		"""
		return glyph data as original vector form
		"""
		if not self._glyph_vectors:
			self._glyph_vectors = {}
		if not index in self._glyph_vectors:
			self._glyph_vectors[index] = self._read_glyph(index)
		return self._glyph_vectors[index]

	def get_glyph_contours(self,index,closed):
		"""
		get glyph vector data and convert to list of contours
		"""
		gv = self.get_glyph_vector(index)
		if(not gv):return

		#splits contours
		contours = []
		c = 0
		newContour = []
		for i,p in enumerate(gv.points):
			newContour.append(p)
			if (i == gv.contoursEnd[c]):
				c += 1
				newContour.append(newContour[0])#add first point
				contours.append(newContour)#add contour to list
				newContour = []

		#add virtual ON point if double OFF sequence is encounter (from cubic to quadratic curve)
		normContours = []
		for cont in contours:
			newCont = []
			p_prec = cont[0]
			for p in cont:
				if (not p.ON_CURVE) and (not p_prec.ON_CURVE):
					newP = GlyphPoint()
					newP.x = (p.x + p_prec.x) / 2.0
					newP.y = (p.y + p_prec.y) / 2.0
					newP.ON_CURVE = True
					newCont.append(newP)
				newCont.append(p)
				p_prec = p

			#make all lines
			normContLine = []
			i = 0
			while (i < len(newCont) - 1):
				#This is a straight line (ON-ON)
				if newCont[i].ON_CURVE and newCont[i + 1].ON_CURVE:
					normContLine.append(GlyphPoint(newCont[i].x,newCont[i].y))
					i+=1
					continue
				#This is a beizer quadratic curve (ON.OFF.ON)
				if newCont[i].ON_CURVE and not newCont[i + 1].ON_CURVE:
					lp = []
					t = 0
					while (t <= 1):
						newP = self.approx_quadratic_bez(newCont[i],newCont[i + 1],newCont[i + 2],t)
						lp.append(newP)
						t += 0.250
					for p in lp:
						normContLine.append(GlyphPoint(p.x,p.y))
					i+=2
					continue
                        if (closed):
                        	normContLine.append(GlyphPoint(newCont[-1].x,newCont[-1].y))

			#add normalized contour to all contours
			normContours.append(normContLine)

		#From inside to outside
		normContours.reverse()
		return normContours

	#Casteljau's Algorithm
	def approx_quadratic_bez(self,  p1, p2, p3, t):
		#p1 = start point
		#p2 = control point
		#p3 = end point
		# Vector addition of P0+P1
		q0_x, q0_y = ((1 - t) * p1.x + t * p2.x,
						(1 - t) * p1.y + t * p2.y)
		q1_x, q1_y = ((1 - t) * p2.x + t * p3.x,
						(1 - t) * p2.y + t * p3.y)
		b_x, b_y = ((1 - t) * q0_x + t * q1_x, (1 - t) * q0_y + t * q1_y)
		nP = GlyphPoint(b_x,b_y)
		return nP

	def _read_array(self, format, offset):
		size = struct.calcsize(format)
		return struct.unpack(format, self._data[offset:offset + size])

	def close(self):
		"""Close the font file.

		This is a good idea, since the entire file is memory mapped in
		until this method is called.  After closing cannot rely on the
		``get_*`` methods.
		"""

		self._data.close()
		os.close(self._fileno)

def _read_table(*entries):
	""" Generic table constructor used for table formats listed at
	end of file."""
	fmt = '>'
	names = []
	for entry in entries:
		name, type = entry.split(':')
		names.append(name)
		fmt += type
	class _table_class:
		size = struct.calcsize(fmt)
		def __init__(self, data, offset):
			items = struct.unpack(fmt, data[offset:offset + self.size])
			self.pairs = zip(names, items)
			for name, value in self.pairs:
				setattr(self, name, value)

		def __repr__(self):
			s = '{' + ', '.join(['%s = %s' % (name, value) \
								  for name, value in self.pairs]) + '}'
			return s

		@staticmethod
		def array(data, offset, count):
			tables = []
			for i in range(count):
				tables.append(_table_class(data, offset))
				offset += _table_class.size
			return tables
	return _table_class


# Table formats (see references)
_read_offset_table = _read_table('scalertype:I',
			'num_tables:H',
			'search_range:H',
			'entry_selector:H',
			'range_shift:H')

_read_table_directory_entry = _read_table('tag:4s',
				'check_sum:I',
				'offset:I',
				'length:I')
_read_head_table = _read_table('version:i',
			'font_revision:i',
			'check_sum_adjustment:L',
			'magic_number:L',
			'flags:H',
			'units_per_em:H',
			'created:Q',
			'modified:Q',
			'x_min:h',
			'y_min:h',
			'x_max:h',
			'y_max:h',
			'mac_style:H',
			'lowest_rec_p_pEM:H',
			'font_direction_hint:h',
			'index_to_loc_format:h',
			'glyph_data_format:h')

_read_OS2_table = _read_table('version:H',
			'x_avg_char_width:h',
			'us_weight_class:H',
			'us_width_class:H',
			'fs_type:H',
			'y_subscript_x_size:h',
			'y_subscript_y_size:h',
			'y_subscript_x_offset:h',
			'y_subscript_y_offset:h',
			'y_superscript_x_size:h',
			'y_superscript_y_size:h',
			'y_superscript_x_offset:h',
			'y_superscript_y_offset:h',
			'y_strikeout_size:h',
			'y_strikeout_position:h',
			's_family_class:h',
			'panose1:B',
			'panose2:B',
			'panose3:B',
			'panose4:B',
			'panose5:B',
			'panose6:B',
			'panose7:B',
			'panose8:B',
			'panose9:B',
			'panose10:B',
			'ul_unicode_range1:L',
			'ul_unicode_range2:L',
			'ul_unicode_range3:L',
			'ul_unicode_range4:L',
			'ach_vend_id:I',
			'fs_selection:H',
			'us_first_char_index:H',
			'us_last_char_index:H',
			's_typo_ascender:h',
			's_typo_descender:h',
			's_typo_line_gap:h',
			'us_win_ascent:H',
			'us_win_descent:H',
			'ul_code_page_range1:L',
			'ul_code_page_range2:L',
			'sx_height:h',
			's_cap_height:h',
			'us_default_char:H',
			'us_break_char:H',
			'us_max_context:H')

_read_kern_header_table = _read_table('version_num:H',
								'n_tables:H')

_read_kern_subtable_header = _read_table('version:H',
								   'length:H',
								   'coverage:H')
_read_kern_subtable_header.horizontal_mask = 0x1
_read_kern_subtable_header.minimum_mask = 0x2
_read_kern_subtable_header.perpendicular_mask = 0x4
_read_kern_subtable_header.override_mask = 0x5
_read_kern_subtable_header.format_mask = 0xf0

_read_kern_subtable_format0 = _read_table('n_pairs:H',
					'search_range:H',
					'entry_selector:H',
					'range_shift:H')
_read_kern_subtable_format0Pair = _read_table('left:H',
						'right:H',
						'value:h')

_read_cmap_header = _read_table('version:H',
				   'num_tables:H')

_read_cmap_encoding_record = _read_table('platform_id:H',
					'encoding_id:H',
					'offset:L')

_read_cmap_format_header = _read_table('format:H',
					 'length:H')
_read_cmap_format4Header = _read_table('format:H',
					'length:H',
					'language:H',
					'seg_count_x2:H',
					'search_range:H',
					'entry_selector:H',
					'range_shift:H')

_read_horizontal_header = _read_table('version:i',
					'Advance:h',
					'Descender:h',
					'LineGap:h',
					'advance_width_max:H',
					'min_left_side_bearing:h',
					'min_right_side_bearing:h',
					'x_max_extent:h',
					'caret_slope_rise:h',
					'caret_slope_run:h',
					'caret_offset:h',
					'reserved1:h',
					'reserved2:h',
					'reserved3:h',
					'reserved4:h',
					'metric_data_format:h',
					'number_of_h_metrics:H')

_read_long_hor_metric = _read_table('advance_width:H',
				  'lsb:h')

_read_naming_table = _read_table('format:H',
				 'count:H',
				 'string_offset:H')

_read_name_record = _read_table('platform_id:H',
				'encoding_id:H',
				'language_id:H',
				'name_id:H',
				'length:H',
				'offset:H')

_read_glyph_size_table = _read_table('numContours:h',
				'xMin:h',
				'yMin:h',
				'xMax:h',
				'yMax:h')

class Glyph:
	def __init__(self):
		self.type = ""
		self.contoursEnd = []
		self.points = []

class GlyphPoint:
	def __init__(self ,x=0.0 , y=0.0):
		self.ON_CURVE = False
		self.x = x
		self.y = y

class GlyphComponent:
	def __init__(self ):
		self.glyphIndex = 0
		#
		destPointIndex = 0
		srcPointIndex = 0
		#Matrix
		self.a = 1
		self.b = 0
		self.c = 0
		self.d = 1
		self.e = 0
		self.f = 0



# -*- coding: ascii -*-
# $Id: CNCEditor.py,v 1.9 2014/10/15 15:04:38 bnv Exp $
#
# Author:       vvlachoudis@gmail.com
# Date: 24-Aug-2014

from __future__ import print_function
try:
	from cStringIO import StringIO
except ImportError:
	from io import StringIO
try:
	import cPickle as pickle
except ImportError:
	import pickle
try:
	from Tkinter import *
	import tkFont
except ImportError:
	from tkinter import *
	import tkinter.font as tkFont

from CNC import Block, CNC
import tkExtra
import re
#import tkDialogs

BLOCK_COLOR   = "LightYellow"
COMMENT_COLOR = "Blue"
DISABLE_COLOR = "Gray"
from CNCCanvas import TAB_COLOR

MAXINT    = 1000000000	# python3 doesn't have maxint

#==============================================================================
# CNC Listbox
#==============================================================================
class CNCListbox(Listbox):
	def __init__(self, master, app, *kw, **kwargs):
		Listbox.__init__(self, master, *kw, **kwargs)
		self.bind("<Button-1>",		self.button1)
		self.bind("<ButtonRelease-1>",	self.release1)
		self.bind("<Double-1>",		self.double)
		self.bind("<Return>",		self.edit)
		self.bind("<KP_Enter>",		self.edit)
		self.bind("<Insert>",		self.insertItem)
		self.bind("<Control-Key-Return>",self.insertItem)
		self.bind("<Control-Key-space>",self.commandFocus)
		self.bind("<Left>",		self.toggleKey)
		self.bind("<Right>",		self.toggleKey)
		self.bind("<Control-Key-d>",	self.clone)
		self.bind("<Control-Key-Up>",	self.orderUp)
		self.bind("<Control-Key-Prior>",self.orderUp)
		self.bind("<Control-Key-Down>",	self.orderDown)
		self.bind("<Control-Key-Next>",	self.orderDown)
		self.bind("<Control-Key-p>",	lambda e : "break")
		self.bind("<Control-Key-n>",	lambda e : "break")
		self.bind("<Control-Key-D>",	self.dump)
		self.bind("<Delete>",		self.deleteBlock)
		self.bind("<BackSpace>",	self.deleteBlock)
		try:
			self.bind("<KP_Delete>",self.deleteBlock)
		except:
			pass

		self.bind("<Control-Key-b>",	self.insertBlock)
		self.bind("<Control-Key-r>",	self.fill)

		self._blockPos = []		# listbox position of each block
		self._items    = []		# each listbox lien which item (bid,lid) shows
		self.app       = app
		self.gcode     = app.gcode
		self.font      = tkFont.nametofont(self.cget("font"))
		self._ystart   = 0
		self._double   = False	# double clicked handled
		self._hadfocus = False
		self.filter    = None

	# ----------------------------------------------------------------------
	def commandFocus(self, event=None):
		self.app.commandFocus(event)
		return "break"

	# ----------------------------------------------------------------------
	# Change the value of a list item
	# and return the value of the old one
	# ----------------------------------------------------------------------
	def set(self, index, value):
		"""Set/Change the value of a list item"""
		try:
			sel = self.selection_includes(index)
			act = self.index(ACTIVE)
			self.delete(index)
		except TclError:
			return
		self.insert(index, value)
		if sel: self.selection_set(index)
		self.activate(act)

	# ----------------------------------------------------------------------
	# Fill listbox with enable items
	# ----------------------------------------------------------------------
	def fill(self, event=None):
		ypos = self.yview()[0]
		act = self.index(ACTIVE)

		#sel = self.curselection()
		items = self.getSelection()
		self.delete(0,END)

		del self._blockPos[:]
		del self._items[:]
		y = 0
		for bi,block in enumerate(self.gcode.blocks):
			if self.filter is not None:
				if not (self.filter in block.name() or \
					self.filter=="enable" and block.enable or
					self.filter=="disable" and not block.enable):
						self._blockPos.append(None)
						continue

			self._blockPos.append(y)
			self.insert(END, block.header())
			self._items.append((bi, None))
			self.itemconfig(END, background=BLOCK_COLOR)
			y += 1
			if not block.enable:
				self.itemconfig(END, foreground=DISABLE_COLOR)
			if not block.expand: continue

			for lj,line in enumerate(block):
				self.insert(END, line)
				y += 1
				if line and line[0] in ("(","%"):
					self.itemconfig(END, foreground=COMMENT_COLOR)
				self._items.append((bi, lj))

		self.select(items)
		#for i in sel: self.selection_set(i)
		self.yview_moveto(ypos)
		self.activate(act)
		self.see(act)

	# ----------------------------------------------------------------------
	# Copy selected items to clipboard
	# ----------------------------------------------------------------------
	def copy(self, event=None):
		sio = StringIO()
		pickler = pickle.Pickler(sio)
		#sio.write(_PLOT_CLIP)
		for block,line in self.getCleanSelection():
			if line is None:
				pickler.dump(self.gcode.blocks[block].dump())
			else:
				pickler.dump(self.gcode.blocks[block][line])
		self.clipboard_clear()
		self.clipboard_append(sio.getvalue())
		return "break"

	# ----------------------------------------------------------------------
	def cut(self, event=None):
		self.copy()
		self.deleteBlock()
		return "break"

	# ----------------------------------------------------------------------
	def paste(self, event=None):
		try: clipboard = self.selection_get(selection='CLIPBOARD')
		except: return

		ypos = self.yview()[0]
		# paste them after the last selected item
		# bid,lid push them to self so it can be accessed from addLines()
		# python3 might fix this with the inner scope
		try:
			self._bid, self._lid = self._items[self.curselection()[-1]]
		except:
			try:
				self._bid, self._lid = self._items[-1]
			except:
				self._bid = 0
				self._lid = None

		selitems = []
		undoinfo = []

		def addLines(lines):
			for line in lines.splitlines():
				# Create a new block
				if self._lid is None:
					self._bid += 1
					if self._bid > len(self.gcode.blocks):
						self._bid = len(self.gcode.blocks)
					self._lid = MAXINT
					block = Block()
					undoinfo.append(self.gcode.addBlockUndo(self._bid,block))
					selitems.append((self._bid, None))
				else:
					block = self.gcode.blocks[self._bid]

				if self._lid == MAXINT:
					self._lid = len(block)
					selitems.append((self._bid, len(block)))
				else:
					self._lid += 1
					selitems.append((self._bid, self._lid))
				undoinfo.append(self.gcode.insLineUndo(self._bid, self._lid, line))

		try:
			# try to unpickle it
			unpickler = pickle.Unpickler(StringIO(clipboard))
			try:
				while True:
					obj = unpickler.load()
					if isinstance(obj,tuple):
						block = Block.load(obj)
						self._bid += 1
						undoinfo.append(self.gcode.addBlockUndo(self._bid, block))
						selitems.append((self._bid,None))
						self._lid = None
					else:
						addLines(obj)
			except EOFError:
				pass
		except pickle.UnpicklingError:
			# Paste as text
			addLines(clipboard)

		if not undoinfo: return

		self.gcode.addUndo(undoinfo)

		self.selection_clear(0,END)
		self.fill()
		self.yview_moveto(ypos)
		self.select(selitems, clear=True)

		#self.selection_set(ACTIVE)
		#self.see(ACTIVE)
		self.winfo_toplevel().event_generate("<<Modified>>")

	# ----------------------------------------------------------------------
	# Clone selected blocks
	# ----------------------------------------------------------------------
	def clone(self, event=None):
		sel = list(map(int,self.curselection()))
		if not sel: return

		ypos = self.yview()[0]
		undoinfo = []
		self.selection_clear(0,END)
		pos = self._items[sel[-1]][0]+1
		blocks = []
		for i in reversed(sel):
			bid, lid = self._items[i]
			if lid is None:
				undoinfo.append(self.gcode.cloneBlockUndo(bid, pos))
				for i in range(len(blocks)): blocks[i] += 1
				blocks.append(pos)
			else:
				undoinfo.append(self.gcode.cloneLineUndo(bid, lid))
		self.gcode.addUndo(undoinfo)

		self.fill()
		self.yview_moveto(ypos)
		if blocks:
			self.selectBlocks(blocks)
			self.activate(self._blockPos[blocks[-1]])
		else:
			self.selection_set(ACTIVE)
		self.see(ACTIVE)
		self.winfo_toplevel().event_generate("<<Modified>>")
		return "break"

	# ----------------------------------------------------------------------
	# Delete selected blocks of code
	# ----------------------------------------------------------------------
	def deleteBlock(self, event=None):
		sel = list(map(int,self.curselection()))
		if not sel: return

		ypos = self.yview()[0]
		undoinfo = []
		for i in reversed(sel):
			bid, lid = self._items[i]
			if isinstance(lid,int):
				undoinfo.append(self.gcode.delLineUndo(bid, lid))
			else:
				undoinfo.append(self.gcode.delBlockUndo(bid))
		self.gcode.addUndo(undoinfo)

		self.selection_clear(0,END)
		self.fill()
		self.yview_moveto(ypos)
		self.selection_set(ACTIVE)
		self.see(ACTIVE)
		self.winfo_toplevel().event_generate("<<Modified>>")

	# ----------------------------------------------------------------------
	# Edit active item
	# ----------------------------------------------------------------------
	def edit(self, event=None):
		active = self.index(ACTIVE)
		txt = self.get(active)
		if event:
			x = event.x
		else:
			x = 0

		ypos = self.yview()[0]
		bid, lid = self._items[active]
		if lid is None:
			txt0 = txt
			txt = self.gcode[bid].name()
			self.set(active, txt)
			edit = tkExtra.InPlaceEdit(self, select=False, bg=self.cget("bg"))
		else:
			edit = tkExtra.InPlaceEdit(self,x=x, select=False, bg=self.cget("bg"))

		if edit.value is None or edit.value==txt:
			if lid is None:
				self.set(active,txt0)
				self.itemconfig(active, background=BLOCK_COLOR)
				if not self.gcode[bid].enable:
					self.itemconfig(active, foreground=DISABLE_COLOR)
			return

		if isinstance(lid,int):
			self.gcode.addUndo(self.gcode.setLineUndo(bid, lid, edit.value))
			self.set(active, edit.value)
			if edit.value and edit.value[0] in ("(","%"):
				self.itemconfig(active, foreground=COMMENT_COLOR)

		else:
			self.gcode.addUndo(self.gcode.setBlockNameUndo(bid, edit.value))
			self.set(active, self.gcode[bid].header())
			self.itemconfig(active, background=BLOCK_COLOR)
			if not self.gcode[bid].enable:
				self.itemconfig(active, foreground=DISABLE_COLOR)

		self.yview_moveto(ypos)
		self.winfo_toplevel().event_generate("<<Modified>>")

	# ----------------------------------------------------------------------
	# return active block id
	# ----------------------------------------------------------------------
	def activeBlock(self):
		active = self.index(ACTIVE)
		if self._items:
			bid, lid = self._items[active]
		else:
			bid = 0
		return bid

	# ----------------------------------------------------------------------
	# Insert a line or a block
	# ----------------------------------------------------------------------
	def insertItem(self, event=None):
		active = self.index(ACTIVE)
		if active is None: return
		if len(self._items)==0 or self._items[active][1] is None:
			self.insertBlock()
		else:
			self.insertLine()

	# ----------------------------------------------------------------------
	# Insert New Block
	# ----------------------------------------------------------------------
	def insertBlock(self, event=None):
		active = self.index(ACTIVE)
		if self._items:
			bid, lid = self._items[active]
			bid += 1
		else:
			bid = 0

		block = Block()
		block.expand = True
		block.append("g0 x0 y0")
		block.append("g1 z0")
		block.append(CNC.zsafe())
		self.gcode.addUndo(self.gcode.addBlockUndo(bid,block))
		self.selection_clear(0,END)
		self.fill()
		# find location of new block
		while active < self.size():
			if self._items[active][0] == bid:
				break
			active += 1
		self.selection_set(active)
		self.see(active)
		self.activate(active)
		self.edit()
		self.winfo_toplevel().event_generate("<<Modified>>")

	# ----------------------------------------------------------------------
	# Insert a new line below cursor
	# ----------------------------------------------------------------------
	def insertLine(self, event=None):
		active = self.index(ACTIVE)
		if active is None: return
		if len(self._items)==0:
			self.insertBlock()
			return

		bid, lid = self._items[active]

		active += 1

		self.insert(active,"")
		self.selection_clear(0,END)
		self.activate(active)
		self.selection_set(active)
		self.see(active)

		edit = tkExtra.InPlaceEdit(self, bg=self.cget("bg"))
		ypos = self.yview()[0]
		self.delete(active)

		if edit.value is None:
			# Cancel and leave
			active -= 1
			self.activate(active)
			self.selection_set(active)
			self.see(active)
			return

		self.insert(active, edit.value)
		self.selection_set(active)
		self.activate(active)
		if edit.value and edit.value[0] in ("(","%"):
			self.itemconfig(active, foreground=COMMENT_COLOR)
		self.yview_moveto(ypos)

		# Add line into code

		# Correct pointers
		if lid is None:
			lid = 0
		else:
			lid += 1
		self.gcode.addUndo(self.gcode.insLineUndo(bid, lid, edit.value))

		self._items.insert(active, (bid, lid))
		for i in range(active+1,len(self._items)):
			b,l = self._items[i]
			if b != bid: break
			if isinstance(l,int):
				self._items[i] = (b,l+1)
		for i in range(bid+1, len(self._blockPos)):
			if self._blockPos[i] is not None:
				self._blockPos[i] += 1	# shift all blocks below by one

		self.winfo_toplevel().event_generate("<<Modified>>")

	# ----------------------------------------------------------------------
	def toggleKey(self,event=None):
		if not self._items: return
		active = self.index(ACTIVE)
		bid,lid = self._items[active]
		if lid is None:
			self.toggleExpand()
		else:
			# Go to header
			self.selection_clear(0,END)
			self.activate(self._blockPos[bid])
			self.selection_set(ACTIVE)
			self.see(ACTIVE)
			self.winfo_toplevel().event_generate("<<ListboxSelect>>")

	# ----------------------------------------------------------------------
	# Button1 clicked
	# ----------------------------------------------------------------------
	def button1(self, event):
		if self._double: return

		# Remember if we had the focus before clicking
		# to be used later in editing
		self._hadfocus = self.focus_get() == self

		# from a single click
		self._ystart = self.nearest(event.y)
		selected = self.selection_includes(self._ystart)
		loc = self._headerLocation(event)
		if loc is None:
			pass
		elif self._headerLocation(event)<2 and selected:
			return "break"	# do not alter selection!

	# ----------------------------------------------------------------------
	# Release button-1. Warning on separation of double or single click or
	# click and drag
	# ----------------------------------------------------------------------
	def release1(self, event):
		if not self._items: return
		if self._double:
			self._double = False
			return

		self._double = False
		active = self.index(ACTIVE)

		# from a single click
		y = self.nearest(event.y)
		self.activate(y)
		if y != self._ystart: return

		loc = self._headerLocation(event)
		if loc is None:
			# Normal line
			if  active==y:
				# In place edit if we had already the focus
				if self._hadfocus:
					self.edit(event)
		elif loc == 0:
			self.toggleExpand()
		elif loc == 1:
			self.toggleEnable()
		return "break"

	# ----------------------------------------------------------------------
	def double(self, event):
		if self._headerLocation(event) == 2:
			self.edit()
			self._double = True
		else:
			self._double = False

	# ----------------------------------------------------------------------
	# Return location where we clicked on header
	#  0 = expand arrow
	#  1 = enable ballot box
	#  2 = name
	# ----------------------------------------------------------------------
	def _headerLocation(self, event):
		if not self._items: return None
		# from a single click
		y = self.nearest(event.y)

		block,line = self._items[y]
		if line is not None: return None

		txt = self.get(y)
		if event.x <= self.font.measure(txt[:2]):
			return 0
		elif event.x <= self.font.measure(txt[:5]):
			return 1
		else:
			return 2

	# ----------------------------------------------------------------------
	# Toggle expand selection
	# ----------------------------------------------------------------------
	def toggleExpand(self, event=None):
		if not self._items: return None
		items  = list(map(int,self.curselection()))
		expand = None
		active = self.index(ACTIVE)
		bactive,lactive = self._items[active]
		blocks = []
		undoinfo = []
		for i in reversed(items):
			bid,lid = self._items[i]
			if lid is not None:
				if bid in blocks: continue
			blocks.append(bid)
			if expand is None: expand = not self.gcode[bid].expand
			undoinfo.append(self.gcode.setBlockExpandUndo(bid, expand))

		if undoinfo:
			self.gcode.addUndo(undoinfo)
			self.selection_clear(0,END)
			self.fill()
			active = self._blockPos[bactive]
			for bid in blocks:
				self.selectBlock(bid)
			self.activate(active)
			self.see(active)

		self.winfo_toplevel().event_generate("<<Status>>",data="Toggled Expand of selected objects")

	# ----------------------------------------------------------------------
	def _toggleEnable(self, enable=None):
		if not self._items: return None
		items    = list(map(int,self.curselection()))
		active   = self.index(ACTIVE)
		ypos     = self.yview()[0]
		undoinfo = []
		blocks   = []
		for i in items:
			bid,lid = self._items[i]
			if lid is not None:
				if bid in blocks: continue
				pos = self._blockPos[bid]
			else:
				pos = i

			blocks.append(bid)
			block = self.gcode[bid]
			if block.name() in ("Header", "Footer"): continue
			if enable is None: enable = not block.enable
			undoinfo.append(self.gcode.setBlockEnableUndo(bid, enable))

			sel = self.selection_includes(pos)
			self.delete(pos)
			self.insert(pos, block.header())
			self.itemconfig(pos, background=BLOCK_COLOR)
			if not block.enable:
				self.itemconfig(pos, foreground=DISABLE_COLOR)
			if sel: self.selection_set(pos)

		if undoinfo:
			self.gcode.calculateEnableMargins()
			self.gcode.addUndo(undoinfo)
			self.activate(active)
			self.yview_moveto(ypos)
			self.winfo_toplevel().event_generate("<<ListboxSelect>>")

	# ----------------------------------------------------------------------
	def enable(self, event=None):
		self._toggleEnable(True)
		self.winfo_toplevel().event_generate("<<Status>>",data="Enabled selected objects")

	# ----------------------------------------------------------------------
	def disable(self, event=None):
		self._toggleEnable(False)
		self.winfo_toplevel().event_generate("<<Status>>",data="Disabled selected objects")

	# ----------------------------------------------------------------------
	# toggle state enable/disable
	# ----------------------------------------------------------------------
	def toggleEnable(self, event=None):
		self._toggleEnable()
		self.winfo_toplevel().event_generate("<<Status>>",data="Toggled Visibility of selected objects")

	# ----------------------------------------------------------------------
	# comment uncomment row
	# ----------------------------------------------------------------------
	def commentRow(self, event=None):
		if not self._items: return
		all_items = self._items
		sel_items = list(map(int,self.curselection()))
		mreg = re.compile("^\((.*)\)$")
		change = False
		for i in sel_items:
			my_item = all_items[i]
			if my_item[1] is not None:
				change = True
				#check for ()
				line = self.gcode[my_item[0]][my_item[1]]
				m = mreg.search(line)
				if m is None:
					self.gcode[my_item[0]][my_item[1]] = "("+line+")"
				else:
					self.gcode[my_item[0]][my_item[1]] = m.group(1)
		if change: self.fill()

	# ----------------------------------------------------------------------
	# splitBlocks
	# ----------------------------------------------------------------------
	def joinBlocks(self, event=None):
		if not self._items: return
		all_items = self._items
		sel_items = list(map(int,self.curselection()))
		change = True
		bl = Block(self.gcode[sel_items[0]].name())
		for bid in sel_items:
			for line in self.gcode[bid]:
				bl.append(line)
			bl.append("( ---------- cut-here ---------- )")
		del bl[-1]
		self.gcode.addUndo(self.gcode.addBlockUndo(bid+1,bl))
		if change: self.fill()
		self.deleteBlock()
		self.winfo_toplevel().event_generate("<<Modified>>")

	# ----------------------------------------------------------------------
	# splitBlocks
	# ----------------------------------------------------------------------
	def splitBlocks(self, event=None):
		if not self._items: return
		all_items = self._items
		sel_items = list(map(int,self.curselection()))
		change = True
		newblocks = []
		for bid in sel_items:
			bl = Block(self.gcode[bid].name())
			for line in self.gcode[bid]:
				if line == "( ---------- cut-here ---------- )":
					#newblocks.append(bl)
					#self.insertBlock(bl)
					self.gcode.addUndo(self.gcode.addBlockUndo(bid+1,bl))
					bl = Block(self.gcode[bid].name())
				else:
					bl.append(line)
		self.gcode.addUndo(self.gcode.addBlockUndo(bid+1,bl))
		#newblocks.append(bl)
		#self.gcode.extend(newblocks)
		if change: self.fill()
		self.deleteBlock()
		self.winfo_toplevel().event_generate("<<Modified>>")

	# ----------------------------------------------------------------------
	# change color of a block
	# ----------------------------------------------------------------------
	def changeColor(self, event=None):
		items = list(map(int,self.curselection()))
		if not items:
			self.winfo_toplevel().event_generate("<<Status>>",data="Nothing is selected")
			return

		# Find initial color
		bid,lid = self._items[items[0]]

		try:
			rgb, color = tkExtra.askcolor(
				title=_("Color"),
				initialcolor=self.gcode[bid].color,
				parent=self)
		except TclError:
			color = None
		if color is None: return

		blocks = []
		undoinfo = []
		for i in reversed(items):
			bid,lid = self._items[i]
			if lid is not None:
				if bid in blocks: continue
			blocks.append(bid)
			oldColor = self.gcode[bid].color
			undoinfo.append(self.gcode.setBlockColorUndo(bid, oldColor))

		if undoinfo:
			self.gcode.addUndo(undoinfo)
			for bid in blocks:
				self.gcode[bid].color = color
			self.winfo_toplevel().event_generate("<<Modified>>")
		self.winfo_toplevel().event_generate("<<Status>>",data="Changed color of block")

	# ----------------------------------------------------------------------
	# Select items in the form of (block, item)
	# ----------------------------------------------------------------------
	def select(self, items, double=False, clear=False, toggle=True):
		if clear:
			self.selection_clear(0,END)
			toggle = False
		first = None

		for bi in items:
			bid,lid = bi
			try:
				block = self.gcode[bid]
			except:
				continue

			if double:
				if block.expand:
					# select whole block
					y = self._blockPos[bid]
				else:
					# select all blocks with the same name
					name = block.nameNop()
					for i,bl in enumerate(self.gcode.blocks):
						if name == bl.nameNop():
							self.selection_set(self._blockPos[i])
					continue

			elif not block.expand or lid is None:
				# select whole block
				y = self._blockPos[bid]

			elif isinstance(lid,int):
				# find line of block
				y = self._blockPos[bid]+1 + lid

			else:
				raise
				#continue

			if y is None: continue

			if toggle:
				select = not self.selection_includes(y)
			else:
				select = True

			if select:
				self.selection_set(y)
				if first is None: first = y
			elif toggle:
				self.selection_clear(y)

		if first is not None:
			self.activate(first)
			self.see(first)

	# ----------------------------------------------------------------------
	# Select whole block lines if expanded
	# ----------------------------------------------------------------------
	def selectBlock(self, bid):
		start = self._blockPos[bid]
		while True:
			bid += 1
			if bid >= len(self._blockPos):
				end = END
				break
			elif self._blockPos[bid] is not None:
				end = self._blockPos[bid]-1
				break
		self.selection_set(start,end)

	# ----------------------------------------------------------------------
	def selectBlocks(self, blocks):
		self.selection_clear(0,END)
		for bid in blocks:
			self.selectBlock(bid)

	# ----------------------------------------------------------------------
	def selectAll(self):
		self.selection_set(0,END)

	# ----------------------------------------------------------------------
	def selectClear(self):
		self.selection_clear(0,END)

	# ----------------------------------------------------------------------
	def selectInvert(self):
		for i in range(self.size()):
			if self.selection_includes(i):
				self.selection_clear(i)
			else:
				self.selection_set(i)

	# ----------------------------------------------------------------------
	# Select all blocks with the same name of the selected layer
	# ----------------------------------------------------------------------
	def selectLayer(self):
		for bid in self.getSelectedBlocks():
			name = self.gcode[bid].nameNop()
			for i,bl in enumerate(self.gcode.blocks):
				if name == bl.nameNop():
					self.selection_set(self._blockPos[i])

	# ----------------------------------------------------------------------
	# Return list of [(blocks,lines),...] currently being selected
	# ----------------------------------------------------------------------
	def getSelection(self):
		return [self._items[int(i)] for i in self.curselection()]

	# ----------------------------------------------------------------------
	# Return all blocks that at least an item is selected
	# ----------------------------------------------------------------------
	def getSelectedBlocks(self):
		blocks = {}
		for i in self.curselection():
			block,line = self._items[int(i)]
			blocks[block] = True
		return list(sorted(blocks.keys()))

	# ----------------------------------------------------------------------
	# Return list of [(blocks,lines),...] currently being selected
	# Filtering all items that the block is also selected
	# ----------------------------------------------------------------------
	def getCleanSelection(self):
		items = [self._items[int(i)] for i in self.curselection()]
		if not items: return items
		blocks = {}
		i = 0
		while i<len(items):
			bid,lid = items[i]
			if lid is None:
				blocks[bid] = True
				i += 1
			elif blocks.get(bid,False):
				del items[i]
			else:
				i += 1
		return items

	# ----------------------------------------------------------------------
	def getActive(self):
		active = self.index(ACTIVE)
		if active is None: return None
		if not self.selection_includes(active):
			try:
				active = self.curselection()[0]
			except:
				return (0,None)
		return self._items[int(active)]

	# ----------------------------------------------------------------------
	# Move selected items upwards
	# ----------------------------------------------------------------------
	def orderUp(self, event=None):
		items = self.getCleanSelection()
		if not items: return
		sel = self.gcode.orderUp(items)
		self.fill()
		self.select(sel,clear=True,toggle=False)
		self.winfo_toplevel().event_generate("<<Modified>>")
		return "break"

	# ----------------------------------------------------------------------
	# Move selected items downwards
	# ----------------------------------------------------------------------
	def orderDown(self, event=None):
		items = self.getCleanSelection()
		if not items: return
		sel = self.gcode.orderDown(items)
		self.fill()
		self.select(sel,clear=True,toggle=False)
		self.winfo_toplevel().event_generate("<<Modified>>")
		return "break"

	# ----------------------------------------------------------------------
	# Invert selected blocks
	# ----------------------------------------------------------------------
	def invertBlocks(self, event=None):
		blocks = self.getSelectedBlocks()
		if not blocks: return
		self.gcode.addUndo(self.gcode.invertBlocksUndo(blocks))
		self.fill()
		# do not send a modified message, no need to redraw
		return "break"

	# ----------------------------------------------------------------------
	# Dump list and code, check for mismatch
	# ----------------------------------------------------------------------
	def dump(self, event=None):
		if not CNC.developer: return
		print("*** LIST ***")
		for i,sel in enumerate(self.get(0,END)):
			print(i,sel.encode("ascii","replace"))

		print("\n*** ITEMS ***")
		for i,item in enumerate(self._items):
			print(i,item)

		print("\n*** CODE ***")
		for i, block in enumerate(self.gcode.blocks):
			print("Block:",i,block.name())
			for j,line in enumerate(block):
				print("   %3d %s"%(j,line))

		print("\nBLOCKPOS=",self._blockPos)



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	Filippo Rivato
# Date:	26 September 2015
# special thanks to my wonderful wife Giulia.

__author__ = "Filippo Rivato"
__email__  = "f.rivato@gmail.com"

__name__ = _("Spirograph")
__version__= "0.0.1"

import math
from CNC import CNC,Block
from ToolsPage import Plugin
from fractions import gcd

#==============================================================================
#Spirograph class
#==============================================================================
class Spirograph:
	def __init__(self,name="Spirograph"):
		self.name = name

	#----------------------------------------------------------------------
	def lcm(self,x,y):
		return (x*y)/gcd(x,y)

	#----------------------------------------------------------------------
	def calc_dots(self,resolution=2*math.pi/360):
		def x():
			return (self.RExt - self.RInt) * math.cos( self.theta ) +\
         self.ROff * math.cos( (self.RExt - self.RInt) / self.RInt * self.theta )

		def y():
			return (self.RExt - self.RInt) * math.sin( self.theta ) -\
         self.ROff * math.sin( (self.RExt - self.RInt) / self.RInt * self.theta )

		while self.theta < (2*self.PI * self.Spins):
			yield (x(), y())
			self.theta += resolution

	#----------------------------------------------------------------------
	def make(self,RExt=50., RInt=33., ROff=13. , Depth=0):
		self.RExt = RExt
		self.RInt = RInt
		self.ROff = ROff

		if RExt>RInt :
			self.Spins = self.lcm(RExt,RInt) / max(RExt,RInt)
		else:
			self.Spins = self.lcm(RExt,RInt) / min(RExt,RInt)

		self.Depth = Depth
		self.PI = math.pi
		self.theta = 0.0

		blocks = []
		block = Block(self.name)

		block.append("(External Radius = %g)"%(self.RExt))
		block.append("(Internal Radius = %g)"%(self.RInt))
		block.append("(Offset Radius = %g)"%(self.ROff))

		xi,yi = zip(*(self.calc_dots()))

		block.append(CNC.zsafe())
		block.append(CNC.grapid(xi[0],yi[0]))

		currDepth = 0.
		stepz = CNC.vars['stepz']
		if stepz==0 : stepz=0.001  #avoid infinite while loop

		while True:
			currDepth -= stepz
			if currDepth < self.Depth : currDepth = self.Depth
			block.append(CNC.zenter(currDepth))
			block.append(CNC.gcode(1, [("f",CNC.vars["cutfeed"])]))
			for x,y in zip(xi,yi):
				block.append(CNC.gline(x,y))
			block.append(CNC.gline(xi[0],yi[0]))
			if currDepth <= self.Depth : break

		block.append(CNC.zsafe())
		blocks.append(block)
		return blocks

#==============================================================================
# Create a sphirograph plot
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Create a spirograph path")
	def __init__(self, master):
		Plugin.__init__(self, master, "Spirograph")
		self.icon  = "spirograph"
		self.group = "Artistic"
		self.variables = [
			("name",      "db" ,    "", _("Name")),
			("RadiusExternal"  ,   "mm" ,    50.0, _("External Radius")),
			("RadiusInternal"  ,   "mm" ,    33.0, _("Internal Radius")),
			("RadiusOffset"  ,   "mm" ,    13.0, _("Offset radius")),
			("Depth"  ,   "mm" ,    0, _("Target Depth"))
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def execute(self, app):
		n = self["name"]
		if not n or n=="default": n="Spirograph"
		spirograph = Spirograph(n)

		blocks = spirograph.make(self.fromMm("RadiusExternal"),
				self.fromMm("RadiusInternal"),
				self.fromMm("RadiusOffset"),
				self.fromMm("Depth"))

		active = app.activeBlock()
		if active==0: active=1
		app.gcode.insBlocks(active, blocks, "Spirograph")
		app.refresh()
		app.setStatus(_("Generated: Spirograph"))

if __name__=="__main__":
	spirograph = Spirograph()
	spirograph.make()



import numpy as np


def load_ply(fileobj):
    """Same as load_ply, but takes a file-like object"""
    def nextline():
        """Read next line, skip comments"""
        while True:
            line = fileobj.readline()
            assert line != ''  # eof
            if not line.startswith('comment'):
                return line.strip()

    assert nextline() == 'ply'
    assert nextline() == 'format ascii 1.0'
    line = nextline()
    assert line.startswith('element vertex')
    nverts = int(line.split()[2])
    # print 'nverts : ', nverts
    assert nextline() == 'property float x'
    assert nextline() == 'property float y'
    assert nextline() == 'property float z'
    line = nextline()

    assert line.startswith('element face')
    nfaces = int(line.split()[2])
    # print 'nfaces : ', nfaces
    assert nextline() == 'property list uchar int vertex_indices'
    line = nextline()
    has_texcoords = line == 'property list uchar float texcoord'
    if has_texcoords:
        assert nextline() == 'end_header'
    else:
        assert line == 'end_header'

    # Verts
    verts = np.zeros((nverts, 3))
    for i in xrange(nverts):
        vals = nextline().split()
        verts[i, :] = [float(v) for v in vals[:3]]
    # Faces
    faces = []
    faces_uv = []
    for i in xrange(nfaces):
        vals = nextline().split()
        assert int(vals[0]) == 3
        faces.append([int(v) for v in vals[1:4]])
        if has_texcoords:
            assert len(vals) == 11
            assert int(vals[4]) == 6
            faces_uv.append([(float(vals[5]), float(vals[6])),
                             (float(vals[7]), float(vals[8])),
                             (float(vals[9]), float(vals[10]))])
            # faces_uv.append([float(v) for v in vals[5:]])
        else:
            assert len(vals) == 4
    return verts, faces, faces_uv


def save_ply(filename, verts, faces):
    with open(filename, 'w') as f:
        f.write('ply\n')
        f.write('format ascii 1.0\n')
        f.write('element vertex %d\n' % verts.shape[0])
        f.write('property float x\n')
        f.write('property float y\n')
        f.write('property float z\n')
        f.write('element face %d\n' % len(faces))
        f.write('property list uchar int vertex_indices\n')
        f.write('end_header\n')
        for i in xrange(verts.shape[0]):
            f.write('%f %f %f\n' % (verts[i, 0], verts[i, 1], verts[i, 2]))
        for i in xrange(len(faces)):
            f.write('3 %d %d %d\n' % (faces[i][0], faces[i][1], faces[i][2]))



#
# Copyright and User License
# ~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright Vasilis.Vlachoudis@cern.ch for the
# European Organization for Nuclear Research (CERN)
#
# Please consult the flair documentation for the license
#
# DISCLAIMER
# ~~~~~~~~~~
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY, OF
# SATISFACTORY QUALITY, AND FITNESS FOR A PARTICULAR PURPOSE
# OR USE ARE DISCLAIMED. THE COPYRIGHT HOLDERS AND THE
# AUTHORS MAKE NO REPRESENTATION THAT THE SOFTWARE AND
# MODIFICATIONS THEREOF, WILL NOT INFRINGE ANY PATENT,
# COPYRIGHT, TRADE SECRET OR OTHER PROPRIETARY RIGHT.
#
# LIMITATION OF LIABILITY
# ~~~~~~~~~~~~~~~~~~~~~~~
# THE COPYRIGHT HOLDERS AND THE AUTHORS SHALL HAVE NO
# LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL,
# CONSEQUENTIAL, EXEMPLARY, OR PUNITIVE DAMAGES OF ANY
# CHARACTER INCLUDING, WITHOUT LIMITATION, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES, LOSS OF USE, DATA OR PROFITS,
# OR BUSINESS INTERRUPTION, HOWEVER CAUSED AND ON ANY THEORY
# OF CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE), PRODUCT
# LIABILITY OR OTHERWISE, ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
#
# Author:	Vasilis.Vlachoudis@cern.ch
# Date:	14-May-2004

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import string

_letters_digits = string.ascii_letters + string.digits
_letters_digits_symbol = _letters_digits + "_."

# abbrev
def abbrev(information, info, l=0):
	"""
	return true if the info is an abbreviation of information
	with minimum length l
	"""
	if l>0:
		length = l
	else:
		length = len(info)

	cond1 = (len(information) >= len(info))
	cond2 = (len(info) >= length)
	cond3 = (information[:len(info)] == info)
	return cond1 and cond2 and cond3

# center
def center(s, l, pad=' '):
	if l<=0: return ""

	i = l - len(s);
	if i==0:
		return s
	elif i < 0:
		i = -i
		a = i // 2
		return s[a:a+l]
	else:
		a = i // 2
		return "%s%s%s"%(pad*a, s, pad*(i-a))

# datatype
def datatype(str, check="N"):
	"""rexx datatype function"""

	try:
		if len(str)==0:
			return check=="X" or check=="B"
	except:
		return check=="X" or check=="B"

	if check=="N":
		return _isnum(str)

	if check=="A":
		return verify(str, _letters_digits)==-1
	elif check=="L":
		return verify(str, string.ascii_lowercase)==-1
	elif check=="M":
		return verify(str, string.ascii_letters)==-1
	elif check=="U":
		return verify(str, string.ascii_uppercase)==-1
	elif check=="O":
		return verify(str, string.octdigits)==-1
	elif check=="X":
		return verify(str, string.hexdigits)==-1
	elif check=="S":
		return (str[0] in string.ascii_letters) and \
			(verify(str[1:], _letters_digits_symbol)==-1)
	else:
		return _isnum(str)

# insert
def insert(new, target, n, pad=" "):
	"""
	insert new string to target as position n padded with pad characters
	"""
	if n==0:
		return new+target
	elif n>len(target):
		return target + pad*(n-len(target)) + new

	return target[0:n] + new + target[n:]

# left
def left(str, length, pad=" "):
	"""return left of string str of length padded with pad chars"""
	if length<len(str):
		return str[0:length]
	else:
		return str + (pad*(length-len(str)))

# translate
def translate(str, tableo=None, tablei=None, pad=" "):
	"""translate string"""
	# If neither input nor output tables, uppercase.
	if tableo==None and tablei==None:
		return str.upper()

	if tableo==None:
		tableo = xrange(0,255)

	if tablei==None:
		tablei = xrange(0,255)

	# The input table defaults to all characters.
	dl = len(tablei)-len(tableo)
	if dl>0:
		tableo += pad*dl
	else:
		tablei += pad*(-dl)

	tbl = string.maketrans(tablei,tableo)
	return str.translate(tbl)

# reverse
def reverse(str):
	"""reverse string"""
	return str[::-1]

# verify
def verify(str,ref,match=0,start=0):
	"""
	return the index of the first character in string that
	is not also in reference. if "Match" is given, then return
	the result index of the first character in string that is in reference
	"""

	if start<0: start = 0
	if start>=len(str): return -1

	for i in range(start,len(str)):
		found = ref.find(str[i])==-1
		if found ^ match:
			return i
	return -1

# xrange
def xrange(start,stop):
	return "".join([chr(x) for x in range(start, stop+1)])

# isnum - return true if string is number
def _isnum(str):
	str = str.strip()

	# accept one sign
	i = 0
	l = len(str)

	if l==0: return False

	if str[i]=='-' or str[i]=='+': i += 1

	# skip spaces after sign
	while i<l and str[i].isspace(): i += 1

	# accept many digits
	if i<l and '0'<=str[i]<='9':
		i += 1
		F  = 1
		while i<l and '0'<=str[i]<='9': i += 1
	else:
		F = 0

	# accept one dot
	if i<l and str[i]=='.':
		i+=1

		# accept many digits
		if i<l and '0'<=str[i]<='9':
			while i<l and '0'<=str[i]<='9': i += 1
		else:
			if not F: return False
	else:
		if not F: return False

	# accept one e/E/d/D
	if i<l and (str[i]=='e' or str[i]=='E' or str[i]=='d' or str[i]=='D'):
		i+=1
		# accept one sign
		if i<l and (str[i]=='-' or str[i]=='+'): i += 1

		# accept many digits
		if i<l and '0'<=str[i]<='9':
			while i<l and '0'<=str[i]<='9': i += 1
		else:
			return False

	if i != l: return False

	return True

if __name__=="__main__":
	from log import say

	say("abbrev")
	assert     abbrev('information','info',4)
	assert     abbrev('information','',0)
	assert not abbrev('information','Info',4)
	assert not abbrev('information','info',5)
	assert not abbrev('information','info ')
	assert     abbrev('information','info',3)
	assert not abbrev('info','information',3)
	assert not abbrev('info','info',5)

	say("center")
	assert center('****',0,'-')      == ''
	assert center('****',8,'-')      == '--****--'
	assert center('****',7,'-')      == '-****--'
	assert center('*****',8,'-')     == '-*****--'
	assert center('*****',7,'-')     == '-*****-'
	assert center('12345678',4,'-')  == '3456'
	assert center('12345678',5,'-')  == '23456'
	assert center('1234567',4,'-')   == '2345'
	assert center('1234567',5,'-')   == '23456'

	say("datatype")
	assert not datatype("")
	assert not datatype("foobar")
	assert not datatype("foo bar")
	assert not datatype("123.456.789")
	assert     datatype("123.456")
	assert not datatype("DeadBeef")
	assert not datatype("Dead Beef")
	assert not datatype("1234ABCD")
	assert     datatype("01001101")
	assert not datatype("0110 1101")
	assert not datatype("0110 101")
	assert     datatype("1324.1234")
	assert     datatype("123")
	assert     datatype("12.3")
	assert     datatype('123.123')
	assert     datatype('123.123E3')
	assert     datatype('123.0000003')
	assert     datatype('123.0000004')
	assert     datatype('123.0000005')
	assert     datatype('123.0000006')
	assert     datatype(' 23')
	assert     datatype(' 23 ')
	assert     datatype('23 ')
	assert     datatype('123.00')
	assert     datatype('123000E-2')
	assert     datatype('123000E+2')
	assert not datatype("A B C")
	assert not datatype("123ABC")
	assert not datatype("123AHC")
	assert     datatype('0.000E-2')
	assert     datatype('0.000E-1')
	assert     datatype('0.000E0')
	assert     datatype('0.000E1')
	assert     datatype('0.000E2')
	assert     datatype('0.000E3')
	assert     datatype('0.000E4')
	assert     datatype('0.000E5')
	assert     datatype('0.000E6')
	assert     datatype('0E-1')
	assert     datatype('0E0')
	assert     datatype('0E1')
	assert     datatype('0E2')
	assert not datatype('+.')
	assert not datatype('++0')

	say("insert")
	assert insert("abc","def",2) == "deabcf"
	assert insert("abc","def",3) == "defabc"
	assert insert("abc","def",5) == "def  abc"
	assert insert("abc","def",5,'*') == "def**abc"

	say("translate")
#	assert translate("Foo Bar"), "FOO BAR"
	assert translate("Foo Bar","","") == "Foo Bar"
#	assert translate("Foo Bar","") == "       "
#	assert translate("Foo Bar",None,None,'*') == "*******"
	assert translate("Foo Bar",xrange(1,255)) == "Gpp!Cbs"
	assert translate("","klasjdf","woieruw") == ""
	assert translate("foobar","abcdef","fedcba") == "aooefr"

	say("verify")
	assert verify('foobar', 'barfo', 0, 0)==-1
	assert verify('foobar', 'barfo', 1, 0)==0
	assert verify('', 'barfo')==-1
	assert verify('foobar', '')==0
	assert verify('foobar', 'barf', 0, 2)==2
	assert verify('foobar', 'barf', 0, 3)==-1
	assert verify('', '')==-1

	say("All Test passed")



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	Filippo Rivato
# Date: 	07 January 2016
# Inspired by: "Death to sharpie" a drawbot by Scott Cooper
# see at http://www.dullbits.com/drawbot/drawbot

__author__ = "Filippo Rivato"
__email__  = "f.rivato@gmail.com"

__name__ = _("Sketch")
__version__= "0.0.1"

import math
# import time
import random
from array import *

from CNC import CNC,Block
from ToolsPage import Plugin


#==============================================================================
# Create sketch
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Create sketch based on picture brightness")
	def __init__(self, master):
		Plugin.__init__(self, master, "Sketch")
		self.icon  = "sketch"
		self.group = "Artistic"

		self.variables = [
			("name",              "db" ,        "", _("Name")),
			("Grundgy","Low,Medium,High,Very High", "Medium",  _("Grundgy, search radius")),
			("Depth"  ,           "mm" ,       0.0, _("Working Depth")),
			("MaxSize"  ,         "mm" ,     250.0, _("Maximum size")),
			("SquiggleTotal" ,   "int" ,       300, _("Squiggle total count")),
			("SquiggleLength",    "mm" ,     400.0, _("Squiggle Length")),
			("DrawBorder",      "bool",      False, _("Draw border")),
			("File"  ,          "file" ,        "", _("Image to process")),
			("Channel","Luminance,Red,Green,Blue" ,"Luminance", _("Channel to analyze")),
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def findFirst(self, pix, scanAll):
		most = 256
		for x in xrange(2,self.imgWidth - 2):
			for y in xrange(2,self.imgHeight - 2):
				val = pix[x,y]

				if val < most:
					most = val
					bestX = x
					bestY = y
				if (val <= self.mostest) and not scanAll:
					self.mostest = most
					return bestX,bestY

		self.mostest = most
		return bestX,bestY

	# ----------------------------------------------------------------------
	def findInRange(self, startX, startY, pix, maxRange):
		xmin = int(max(2, startX - maxRange))
		xmax = int(min(self.imgWidth - 2, startX + maxRange))
		ymin = int(max(2, startY - maxRange))
		ymax = int(min(self.imgHeight - 2, startY + maxRange))

		bestX = startX
		bestY = startY
		distance=1

		most = 256
		for x in xrange(xmin,xmax):
			for y in xrange(ymin,ymax):
				distance = math.sqrt((startX - x)**2 + (startY -y)**2)
				if(distance > maxRange):
					continue
				val = pix[x,y]
				val += (random.random()*2.)  #avoid ugly straight lines, steal time
				if val < most:
					most = val
					bestX = x
					bestY = y
				if most <= self.mostest:
					self.mostest = most
					return bestX, bestY, distance
		return bestX, bestY, distance

	def fadePixel(self, x, y, pix):
		pix[x,y] +=40
		pix[x+1,y] +=16
		pix[x-1,y] +=16
		pix[x,y+1] +=16
		pix[x,y-1] +=16
		pix[x+1,y+1] +=8
		pix[x-1,y-1] +=8
		pix[x-1,y+1] +=8
		pix[x+1,y-1] +=8

		#pix[x-2,y-2] +=4
		pix[x-2,y-1] +=4
		pix[x-2,y-0] +=4
		pix[x-2,y+1] +=4
		#pix[x-2,y+2] +=4

		#pix[x+2,y-2] +=4
		pix[x+2,y-1] +=4
		pix[x+2,y-0] +=4
		pix[x+2,y+1] +=4
		#pix[x+2,y+2] +=4

		#pix[x-2,y-2] +=4
		pix[x-1,y-2] +=4
		pix[x-0,y-2] +=4
		pix[x+1,y+2] +=4
		#pix[x+2,y+2] +=4

		#pix[x-2,y+2] +=4
		pix[x-1,y+2] +=4
		pix[x-0,y+2] +=4
		pix[x+1,y+2] +=4
		#pix[x+2,y+2] +=4



	# ----------------------------------------------------------------------
	def execute(self, app):
		try:
			from PIL import Image
		except:
			app.setStatus(_("Sketch abort: This plugin requires PIL/Pillow to read image data"))
			return

		n = self["name"]
		if not n or n=="default": n="Sketch"

		#Calc desired size
		grundgy =self["Grundgy"]
		maxSize = self["MaxSize"]
		squiggleTotal  = self["SquiggleTotal"]
		squiggleLength = self["SquiggleLength"]
		depth = self["Depth"]
		drawBorder = self["DrawBorder"]
		channel = self["Channel"]

		radius = 1
		if grundgy == "Low":
			radius = 2
		elif grundgy == "Medium":
			radius = 3
		elif grundgy == "High":
			radius = 6
		elif grundgy == "Very High":
			radius = 9

		#Check parameters
		if maxSize < 1:
			app.setStatus(_("Sketch abort: Too small to draw anything!"))
			return

		if squiggleTotal < 1:
			app.setStatus(_("Sketch abort: Please let me draw at least 1 squiggle"))
			return
			
		if squiggleLength <= 0:
			app.setStatus(_("Sketch abort: Squiggle Length must be > 0"))
			return

		fileName = self["File"]
		try:
			img = Image.open(fileName)
		except:
			app.setStatus(_("Sketch abort: Can't read image file"))
			return

		#Create a scaled image to work faster with big image and better with small ones
		iWidth,iHeight = img.size
		resampleRatio = 800.0 / iHeight
		img = img.resize((int(iWidth *resampleRatio) ,int(iHeight * resampleRatio)), Image.ANTIALIAS)
		if channel == 'Blue':
			img = img.convert('RGB')
			img = img.split()[0]
		elif channel == 'Green':
			img = img.convert('RGB')
			img = img.split()[1]
		elif channel == 'Red':
			img = img.convert('RGB')
			img = img.split()[2]
		else:
			img = img.convert ('L') #to calculate luminance

		img = img.transpose(Image.FLIP_TOP_BOTTOM) #ouput correct image
		pix = img.load()

		#Get image size
		self.imgWidth, self.imgHeight =  img.size
		self.ratio = 1
		if (iWidth > iHeight):
			self.ratio = maxSize / float(self.imgWidth)
		else:
			self.ratio = maxSize / float(self.imgHeight)

		#Init blocks
		blocks = []

		#Border block
		block = Block("%s-border"%(self.name))
		block.enable = drawBorder
		block.append(CNC.zsafe())
		block.append(CNC.grapid(0,0))
		block.append(CNC.zenter(depth))
		block.append(CNC.gcode(1, [("f",CNC.vars["cutfeed"])]))
		block.append(CNC.gline(self.imgWidth * self.ratio, 0))
		block.append(CNC.gline(self.imgWidth * self.ratio, self.imgHeight*self.ratio))
		block.append(CNC.gline(0, self.imgHeight*self.ratio))
		block.append(CNC.gline(0,0))
		blocks.append(block)

		#Draw block
		block = Block(self.name)
		block.append("(Sketch size W=%d x H=%d x distance=%d)" %
			 (self.imgWidth * self.ratio  , self.imgHeight * self.ratio  , depth))
		block.append("(Channel = %s)" %(channel))
		#choose a nice starting point
		x = self.imgWidth / 4.
		y = self.imgHeight / 4.

		#First round search in all image
		self.mostest = 256
		x,y = self.findFirst(pix, True)

		#startAll = time.time()
		for c in range(squiggleTotal):
			#print c,x,y
			#start = time.time()
			x,y = self.findFirst(pix, False)
			#print 'Find mostest: %f' % (time.time() - start)
			#move there
			block.append(CNC.zsafe())
			block.append(CNC.grapid(x*self.ratio, y*self.ratio))
			#tool down
			block.append(CNC.zenter(depth))
			#restore cut/draw feed
			block.append(CNC.gcode(1, [("f",CNC.vars["cutfeed"])]))

			#start = time.time()
			s = 0
			while (s < squiggleLength):
				x,y,distance = self.findInRange(x, y, pix, radius)
				s+= max(1,distance*self.ratio)  #add traveled distance
				#move there
				block.append(CNC.gline(x*self.ratio,y*self.ratio))
				self.fadePixel(x, y, pix) #adjustbrightness int the bright map
			#tool up
			block.append(CNC.zsafe())
			#print 'Squiggle: %f' % (time.time() - start)

		#Gcode Zsafe
		block.append(CNC.zsafe())
		blocks.append(block)
		active = app.activeBlock()
		app.gcode.insBlocks(active, blocks, "Sketch")
		app.refresh()
		app.setStatus(_("Generated Sketch size W=%d x H=%d x distance=%d, Total length:%d") %
			(self.imgWidth*self.ratio  , self.imgHeight*self.ratio , depth, squiggleTotal*squiggleLength))
		#img.save('test.png')
		#print 'Time: %f' % (time.time() - startAll)
